/*
    Program...: ComponentPaletteUpdate.prg
    Programmer: Ken Mayer
    Date......: April 18, 2002
    Updated...: May 9, 2002 -- to use dBObj.dll instead
                               of StockClasses.dll
                January 23, 2003 -- 
                               to use, if in dBASE Plus,
                               some new images from creg.dll
                               in a few cases. Should work
                               fine in older versions, use
                               of #IF/#ENDIF and check for
                               __version__ ...
                Sept., 2003 -- Modified to use StockClasses.dll
                               again, as Rick modified the DLL ...
                October 23, 2003 --
                               Rick Miller provided some code
                               to expand out the dUFLP source
                               alias for the purpose of finding 
                               the DLL and displaying everything
                               properly.
                July, 2004  -- Modified to add buttons and toolbars
                               in new .cc file: pCustButt.cc
                Sept. 2005  -- Added HTMLLinkText.cc, 
                               DateCalendar.cc and some of the
                               calendar classes that weren't
                               already in here ...
                June 1, 2007 -- Modified to deal with multi-language
                               issues for CREG009.txt ... and to
                               allow for being called from 
                               SETUP.PRG.

                August, 2012 -- Difficulty with current versions of dBASE 
                                (2.7 and later) and Windows Vista/Windows
                                7 (and one presumes later). UAC Paths 
                                throw things off, and these versions default
                                to using UAC Paths. 
                                
                January, 2017 -- added a couple new components to the 
                                table (kmDateWithCal.cc) ...
                                
                November, 2018 -- updates because in some version
                                of dBASE, probably 9, the path
                                to the table creg0009.dbf has
                                shifted to a new location. 
                                
                April, 2020 -- added some new controls ...

    Notes.....: The purpose of this program is to update
                the table used by the form and report designers
                to display components from the dUFLP on tabs
                of their own on the component palette. This is
                done based on the information in the text file
                in the dUFLP: CREG009.TXT

                NOTE: This does *NOT* automatically add items
                to the component palette, you still must
                open the files you want to use, either with:
                   set procedure to :dUFLP:somecc.cc 
                or
                   set procedure to c:\duflp\somecc.cc 
                
                All that this code does is update the table used
                by the form and report designers, adding information
                that references controls contained in the dUFLP.
                For more details, see the file: creg0009.txt
                in the :dUFLP: source alias ...

                I tried to get all the UI controls from the 
                various .CC files in the dUFLP, if I missed
                something, feel free to let me know.

   Usage......: do :dUFLP:ComponentPaletteUpdate.prg

   Misc. Notes: If you wish, you can copy code from here to 
                do the same for your own components ...
                I've attempted to document this in a way
                that hopefully it will be obvious what you
                need ...

                This uses the resource.dll and creg.dll files 
                that ship with dBASE, and Rick Miller's 
                StockClasses.dll file to get the images
                used on the component palette ...
*/
parameter bCalled
if pCount() == 0 or empty( argvector(1) ) or type( "argvector(1)" ) # "L"
   bCalled = false
endif

// first, get the path to the file ...
local cPath, cLang, cTable, cReg

// We need to see if this version of dBASE is using 
// the UAC settings:
if type( "_app.useUACPaths" ) # "L"
   _app.useUACPaths = false // create custom property
endif

// file location will not be correct if UACPaths are in use. 
// Suggestion, copy from default location to another location, then modify,
// at end of routine, copy table to any others ... (copy to ProgramData ...,
// modify, then copy to user paths ...)

// version 9 or later of dBASE, this hasn't changed:
if _app.useUACPaths
   cPath = left( _app.currentUserPath, len( _app.currentUserPath ) - 3) + "Designer\Form"
else // not using UACPaths:
   cPath = _dBWinHome+"designer\form"
endif // _app.useUACPaths


cLang = _app.Language
do case
   case cLang == "EN"
        cTable = "creg0009.dbf"
   case cLang == "DE" // German
        cTable = "creg0007.dbf"
   case cLang == "ES" // Spanish
        cTable = "creg000A.dbf"
   case cLang == "IT" // Italian
        cTable = "creg0010.dbf"
   case cLang == "JA" // Japanese
        cTable = "creg0011.dbf"
   otherwise
        cTable = "creg0009.dbf"
endcase
cReg = cPath+"\"+cTable

// cReg path problem ...:
if not file( cReg )
   msgbox( "Cannot do anything:  "+chr(13)+;
           cReg+chr(13)+;
           "does not exist.", "Can't Do It!", 16 )
   return
endif

// same issue here ... 

// resource.dll path:
cResource = home()+"resource.dll"

// StockClasses.dll path -- need to get path for dUFLP folder:
cKey = _app.SourceAliases.firstKey
lExists = false
for i = 1 to _app.SourceAliases.count()
    if cKey.toLowerCase() == "duflp"
       lExists := true
       // Modified code by Rick Miller -- this code
       // expands the path out so that dBASE
       // can find the information, and so it will be
       // displayed in the component palette properly:
       // October 24, 2003
       cStock = _app.SourceAliases[cKey]+"\StockClasses.dll"
       exit
    endif
    cKey = _app.SourceAliases.nextKey( cKey )
next

if not lExists
   // see if the file "StockClasses.dll" is in the current
   // folder -- if we're running this from the SETUP.PRG file,
   // then the dUFLP source alias may not currently exist,
   // but that program is running from the dUFLP folder, so
   // the StockClasses.dll file should exist in the dUFLP
   // folder, and we should be okay:
   cStock = set("dire")+"\StockClasses.dll"
   // if not:
   if not file( cStock )
      msgbox( "Cannot do anything: dUFLP Source Alias doesn't exist,"+chr(13)+;
              "and/or cannot find '"+cStock+"' file ...",;
              "Can't Do It!", 16 )
      return
   endif
endif   

// before starting, let's check for dUFLP items in there,
// and if they exist, delete 'em (so we don't have double
// entries, and so on ...) -- doing this in quick'n'dirty
// XDML ...
use ( cReg ) excl
delete for "dUFLP"     $ category
delete for "pToolbars" $ category
delete for "pData"     $ category
delete for "pMisc"     $ category
set safety off
pack
set safety on
use

// set up query:
qCReg = new query( [select * from "]+cReg+["] )
// rowset:
rCReg = qCReg.rowset
// fields:
fCReg = rCReg.fields

// to do this, we need to simply add a whole series of rows, one for
// each component we're adding to the table. If handled properly, 
// this will create entries useful for displaying these controls,
// with our own images, and our own tabs:
// Fields are: ClassName
//             FileName (containing image)
//             BitMapID (id number in image)
//             XOFFSET -- only if needed
//             YOFFSET -- only if needed
//             Width (pixels)
//             Height (pixels)
//             Category -- the tab name!!
//             Order -- sequence that they appear on the tab
//             Designer (F = Form, R = Report, N = No palette entry (don't display)


/*
    Buttons and controls in CUSTBUTT.CC
*/

// height of images from Resource.dll
#DEFINE RWIDTH  20
#DEFINE RHEIGHT 20
// height of images from StockClasses.dll by Rick Miller
#DEFINE SWIDTH  16
#DEFINE SHEIGHT 16

// Ken's Toolbar
rCReg.beginAppend()
fCReg["ClassName"].value := "KensToolBar"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Toolbars"
fCReg["Order"].value     := 1
fCReg["Designer"].value  := "F"
rCReg.save()

// pKen's Toolbar
rCReg.beginAppend()
fCReg["ClassName"].value := "pKensToolBar"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pToolbars"
fCReg["Order"].value     := 1
fCReg["Designer"].value  := "F"
rCReg.save()

// Ken's Toolbar2
rCReg.beginAppend()
fCReg["ClassName"].value := "kmToolBar2"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Toolbars"
fCReg["Order"].value     := 2
fCReg["Designer"].value  := "F"
rCReg.save()

// pKen's Toolbar2
rCReg.beginAppend()
fCReg["ClassName"].value := "pkmToolBar2"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pToolbars"
fCReg["Order"].value     := 2
fCReg["Designer"].value  := "F"
rCReg.save()


// NavigateButtons
rCReg.beginAppend()
fCReg["ClassName"].value := "NavigateButtons"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Toolbars"
fCReg["Order"].value     := 3
fCReg["Designer"].value  := "F"
rCReg.save()

// pNavigateButtons
rCReg.beginAppend()
fCReg["ClassName"].value := "pNavigateButtons"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pToolbars"
fCReg["Order"].value     := 3
fCReg["Designer"].value  := "F"
rCReg.save()

// EditButtons
rCReg.beginAppend()
fCReg["ClassName"].value := "EditButtons"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Toolbars"
fCReg["Order"].value     := 4
fCReg["Designer"].value  := "F"
rCReg.save()

// pEditButtons
rCReg.beginAppend()
fCReg["ClassName"].value := "pEditButtons"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pToolbars"
fCReg["Order"].value     := 4
fCReg["Designer"].value  := "F"
rCReg.save()

// EditButtons2
rCReg.beginAppend()
fCReg["ClassName"].value := "EditButtons2"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Toolbars"
fCReg["Order"].value     := 5
fCReg["Designer"].value  := "F"
rCReg.save()

// pEditButtons2
rCReg.beginAppend()
fCReg["ClassName"].value := "pEditButtons2"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pToolbars"
fCReg["Order"].value     := 5
fCReg["Designer"].value  := "F"
rCReg.save()


// ClipBoardButtons
rCReg.beginAppend()
fCReg["ClassName"].value := "ClipBoardButtons"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Toolbars"
fCReg["Order"].value     := 6
fCReg["Designer"].value  := "F"
rCReg.save()

// pClipBoardButtons
rCReg.beginAppend()
fCReg["ClassName"].value := "pClipBoardButtons"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pToolbars"
fCReg["Order"].value     := 6
fCReg["Designer"].value  := "F"
rCReg.save()


// FindButtons
rCReg.beginAppend()
fCReg["ClassName"].value := "FindButtons"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Toolbars"
fCReg["Order"].value     := 7
fCReg["Designer"].value  := "F"
rCReg.save()

// pFindButtons
rCReg.beginAppend()
fCReg["ClassName"].value := "pFindButtons"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pToolbars"
fCReg["Order"].value     := 7
fCReg["Designer"].value  := "F"
rCReg.save()

// ExitButtons
rCReg.beginAppend()
fCReg["ClassName"].value := "ExitButtons"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Toolbars"
fCReg["Order"].value     := 8
fCReg["Designer"].value  := "F"
rCReg.save()

// pExitButtons
rCReg.beginAppend()
fCReg["ClassName"].value := "pExitButtons"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 6
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pToolbars"
fCReg["Order"].value     := 8
fCReg["Designer"].value  := "F"
rCReg.save()

// --- Put the buttons in now, as a separate tab:

// BaseSpeedButton -- don't put on palette
rCReg.beginAppend()
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["ClassName"].value := "BaseSpeedButton"
fCReg["Designer"].value  := "N"
rCReg.save()

// pBaseSpeedButton -- don't put on palette
rCReg.beginAppend()
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["ClassName"].value := "pBaseSpeedButton"
fCReg["Designer"].value  := "N"
rCReg.save()

// kmTopButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmTopButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 2
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 1
fCReg["Designer"].value  := "F"
rCReg.save()

// pTopButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pTopButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 2
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 1
fCReg["Designer"].value  := "F"
rCReg.save()

// kmPreviousButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmPreviousButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 3
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 2
fCReg["Designer"].value  := "F"
rCReg.save()

// pPreviousButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pPreviousButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 3
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 2
fCReg["Designer"].value  := "F"
rCReg.save()

// kmNextButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmNextButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 4
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 3
fCReg["Designer"].value  := "F"
rCReg.save()

// pNextButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pNextButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 4
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 3
fCReg["Designer"].value  := "F"
rCReg.save()

// kmEndButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmEndButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 5
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 4
fCReg["Designer"].value  := "F"
rCReg.save()

// pEndButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pEndButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 5
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 4
fCReg["Designer"].value  := "F"
rCReg.save()

// kmAddButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmAddButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 12
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 5
fCReg["Designer"].value  := "F"
rCReg.save()

// pAddButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pAddButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 12
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 5
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCarryButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCarryButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 12
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 6
fCReg["Designer"].value  := "F"
rCReg.save()

// pCarryButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pCarryButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 12
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 6
fCReg["Designer"].value  := "F"
rCReg.save()

// kmEditButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmEditButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 14
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 7
fCReg["Designer"].value  := "F"
rCReg.save()

// pEditButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pEditButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 14
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 7
fCReg["Designer"].value  := "F"
rCReg.save()

// kmSaveButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmSaveButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 15
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 8
fCReg["Designer"].value  := "F"
rCReg.save()

// pSaveButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pSaveButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 15
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 8
fCReg["Designer"].value  := "F"
rCReg.save()

// kmDeleteButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmDeleteButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 13
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 9
fCReg["Designer"].value  := "F"
rCReg.save()

// pDeleteButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pDeleteButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 13
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 9
fCReg["Designer"].value  := "F"
rCReg.save()

// kmAbandonButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmAbandonButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 16
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 10
fCReg["Designer"].value  := "F"
rCReg.save()

// pAbandonButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pAbandonButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 16
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 10
fCReg["Designer"].value  := "F"
rCReg.save()

// kmFilterButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmFilterButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 18
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 11
fCReg["Designer"].value  := "F"
rCReg.save()

// pFilterButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pFilterButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 18
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 11
fCReg["Designer"].value  := "F"
rCReg.save()

// kmLocateButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmLocateButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 17
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 12
fCReg["Designer"].value  := "F"
rCReg.save()

// pLocateButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pLocateButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 17
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 12
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCutButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCutButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 2030
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 13
fCReg["Designer"].value  := "F"
rCReg.save()

// pCutButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pCutButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 2030
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 13
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCopyButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCopyButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 2040
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 14
fCReg["Designer"].value  := "F"
rCReg.save()

// pCopyButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pCopyButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 2040
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 14
fCReg["Designer"].value  := "F"
rCReg.save()

// kmPasteButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmPasteButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 2050
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 15
fCReg["Designer"].value  := "F"
rCReg.save()

// pPasteButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pPasteButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 2050
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 15
fCReg["Designer"].value  := "F"
rCReg.save()

// kmRowState
rCReg.beginAppend()
fCReg["ClassName"].value := "kmRowState"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 28
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Data Buttons"
fCReg["Order"].value     := 16
fCReg["Designer"].value  := "F"
rCReg.save()

// pRowState
rCReg.beginAppend()
fCReg["ClassName"].value := "pRowState"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 28
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pData Buttons"
fCReg["Order"].value     := 16
fCReg["Designer"].value  := "F"
rCReg.save()

// -- Misc Buttons:

// kmCloseButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCloseButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 1005
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 1
fCReg["Designer"].value  := "F"
rCReg.save()

// pCloseButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pCloseButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 1005
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 1
fCReg["Designer"].value  := "F"
rCReg.save()

// kmYesButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmYesButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 36
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 2
fCReg["Designer"].value  := "F"
rCReg.save()

// pYesButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pYesButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 36
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 2
fCReg["Designer"].value  := "F"
rCReg.save()

// kmNoButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmNoButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 24
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 3
fCReg["Designer"].value  := "F"
rCReg.save()

// pNoButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pNoButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 24
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 3
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCancelButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCancelButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 28
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 4
fCReg["Designer"].value  := "F"
rCReg.save()

// pCancelButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pCancelButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 28
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 4
fCReg["Designer"].value  := "F"
rCReg.save()

// kmQuestionButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmQuestionButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 32
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 5
fCReg["Designer"].value  := "F"
rCReg.save()

// pQuestionButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pQuestionButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 32
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 5
fCReg["Designer"].value  := "F"
rCReg.save()

// kmLightningButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmLightningButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 108
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 5
fCReg["Designer"].value  := "F"
rCReg.save()

// pLightningButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pLightningButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 108
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 5
fCReg["Designer"].value  := "F"
rCReg.save()

// kmInfoButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmInfoButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 112
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 7
fCReg["Designer"].value  := "F"
rCReg.save()

// pInfoButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pInfoButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 112
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 7
fCReg["Designer"].value  := "F"
rCReg.save()

// kmFromDiskButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmFromDiskButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 1410
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 8
fCReg["Designer"].value  := "F"
rCReg.save()

// pFromDiskButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pFromDiskButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 1410
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 8
fCReg["Designer"].value  := "F"
rCReg.save()

// kmToDiskButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmToDiskButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 1430
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 9
fCReg["Designer"].value  := "F"
rCReg.save()

// pToDiskButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pToDiskButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 1430
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 9
fCReg["Designer"].value  := "F"
rCReg.save()

// kmOpenFolderButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmOpenFolderButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 137
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 10
fCReg["Designer"].value  := "F"
rCReg.save()

// pOpenFolderButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pOpenFolderButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 137
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 10
fCReg["Designer"].value  := "F"
rCReg.save()

// kmWrenchButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmWrenchButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 144
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 11
fCReg["Designer"].value  := "F"
rCReg.save()

// pWrenchButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pWrenchButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 144
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 11
fCReg["Designer"].value  := "F"
rCReg.save()

// kmPrintButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmPrintButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 857
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 12
fCReg["Designer"].value  := "F"
rCReg.save()

// kmPrintButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pPrintButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 857
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 12
fCReg["Designer"].value  := "F"
rCReg.save()

// kmSearchButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmSearchButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 858
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 13
fCReg["Designer"].value  := "F"
rCReg.save()

// pSearchButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pSearchButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 858
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 13
fCReg["Designer"].value  := "F"
rCReg.save()

// kmSearchAgainButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmSearchAgainButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 860
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 14
fCReg["Designer"].value  := "F"
rCReg.save()

// pSearchAgainButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pSearchAgainButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 860
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 14
fCReg["Designer"].value  := "F"
rCReg.save()

// kmReplaceButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmReplaceButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 859
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 15
fCReg["Designer"].value  := "F"
rCReg.save()

// pReplaceButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pReplaceButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 859
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 15
fCReg["Designer"].value  := "F"
rCReg.save()

// kmSearchButton2
rCReg.beginAppend()
fCReg["ClassName"].value := "kmSearchButton2"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 858
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 16
fCReg["Designer"].value  := "F"
rCReg.save()

// pSearchButton2
rCReg.beginAppend()
fCReg["ClassName"].value := "pSearchButton2"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 858
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 16
fCReg["Designer"].value  := "F"
rCReg.save()

// kmExitButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmExitButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 1005
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 17
fCReg["Designer"].value  := "F"
rCReg.save()

// pExitButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pExitButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 1005
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 17
fCReg["Designer"].value  := "F"
rCReg.save()

// kmLightbulbButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmLightBulbButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 2020
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 18
fCReg["Designer"].value  := "F"
rCReg.save()

// pLightbulbButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pLightBulbButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 2020
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 18
fCReg["Designer"].value  := "F"
rCReg.save()

// kmFirstPageButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmFirstPageButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 2
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 19
fCReg["Designer"].value  := "F"
rCReg.save()

// pFirstPageButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pFirstPageButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 2
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 19
fCReg["Designer"].value  := "F"
rCReg.save()

// kmNextPageButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmNextPageButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 4
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 20
fCReg["Designer"].value  := "F"
rCReg.save()

// pNextPageButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pNextPageButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 4
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 20
fCReg["Designer"].value  := "F"
rCReg.save()

// kmPreviousPageButton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmPreviousPageButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 3
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Misc. Buttons"
fCReg["Order"].value     := 21
fCReg["Designer"].value  := "F"
rCReg.save()

// pPreviousPageButton
rCReg.beginAppend()
fCReg["ClassName"].value := "pPreviousPageButton"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 3
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "pMisc. Buttons"
fCReg["Order"].value     := 21
fCReg["Designer"].value  := "F"
rCReg.save()

/*
    Form Controls found in FormCntl.cc
*/
// kmFormTitle
rCReg.beginAppend()
fCReg["ClassName"].value := "kmFormTitle"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 1
fCReg["Designer"].value  := "F"
rCReg.save()

// kmVerticalFormTitle1
rCReg.beginAppend()
fCReg["ClassName"].value := "kmVerticalFormTitle1"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 2
fCReg["Designer"].value  := "F"
rCReg.save()

// kmVerticalFormTitle2
rCReg.beginAppend()
fCReg["ClassName"].value := "kmVerticalFormTitle2"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 3
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustText
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustText"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 4
fCReg["Designer"].value  := "F"
rCReg.save()

// kmOtherCustText
rCReg.beginAppend()
fCReg["ClassName"].value := "kmOtherCustText"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 5
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustWrapText
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustWrapText"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 6
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustLabel
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustLabel"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 10
fCReg["BitMapID"].value  := 3038
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 7
fCReg["Designer"].value  := "F"
rCReg.save()

// kmOtherCustLabel
rCReg.beginAppend()
fCReg["ClassName"].value := "kmOtherCustLabel"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 10
fCReg["BitMapID"].value  := 3038
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 8
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustEntryField
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustEntryField"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 9
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustDateEntryField
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustDateEntryField"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 10
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustDateEntryField2
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustDateEntryField2"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 11
fCReg["Designer"].value  := "F"
rCReg.save()

// kmDisableEntryField
rCReg.beginAppend()
fCReg["ClassName"].value := "kmDisableEntryField"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 12
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustSpinbox
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustSpinbox"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 9
fCReg["BitMapID"].value  := 3033
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 13
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustDateSpinbox
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustDateSpinbox"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 9
fCReg["BitMapID"].value  := 3033
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 14
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustDateSpinbox2
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustDateSpinbox2"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 9
fCReg["BitMapID"].value  := 3033
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 15
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustEditor
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustEditor"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 8
fCReg["BitMapID"].value  := 3010
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 16
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustListbox
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustListbox"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 4
fCReg["BitMapID"].value  := 3017
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 17
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustCombobox
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustCombobox"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 5
fCReg["BitMapID"].value  := 3006
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 18
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustComboboxStyle1
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustComboboxStyle1"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 5
fCReg["BitMapID"].value  := 3006
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 19
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustComboboxStyle0
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustComboboxStyle0"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 5
fCReg["BitMapID"].value  := 3006
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 20
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustCheckbox
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustCheckbox"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 7
fCReg["BitMapID"].value  := 3005
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 21
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustRadio
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustRadio"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 6
fCReg["BitMapID"].value  := 3026
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 22
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustPushbutton
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustPushbutton"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 24
fCReg["BitMapID"].value  := 3024
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 23
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustRectangle
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustRectangle"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 24
fCReg["BitMapID"].value  := 3027
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 24
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustRectangle2
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustRectangle2"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 24
fCReg["BitMapID"].value  := 3027
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 25
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustLine
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustLine"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 11
fCReg["BitMapID"].value  := 3016
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 26
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustContainer
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustContainer"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 19
fCReg["BitMapID"].value  := 3007
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 27
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustContainer2
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustContainer2"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 19
fCReg["BitMapID"].value  := 3007
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 28
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustNotebook
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustNotebook"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 16
fCReg["BitMapID"].value  := 3020
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 29
fCReg["Designer"].value  := "F"
rCReg.save()

// kmMultiRowsetNotebook
rCReg.beginAppend()
fCReg["ClassName"].value := "kmMultiRowsetNotebook"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 16
fCReg["BitMapID"].value  := 3020
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 30
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustTabbox
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustTabbox"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 17
fCReg["BitMapID"].value  := 3036
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 31
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustImage
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustImage"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 14
fCReg["BitMapID"].value  := 3015
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 32
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustGrid
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustGrid"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 22
fCReg["BitMapID"].value  := 3013
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 33
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustEditGrid
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustEditGrid"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 22
fCReg["BitMapID"].value  := 3013
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 34
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustGrid2
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustGrid2"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 22
fCReg["BitMapID"].value  := 3013
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 35
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustEditGrid2
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustEditGrid2"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 22
fCReg["BitMapID"].value  := 3013
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 36
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustSlider
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustSlider"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 20
fCReg["BitMapID"].value  := 3032
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 37
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustHorizontalSlider
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustHorizontalSlider"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 20
fCReg["BitMapID"].value  := 3032
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 38
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustVerticalSlider
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustVerticalSlider"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 20
fCReg["BitMapID"].value  := 3032
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 39
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCustProgress
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCustProgress"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 18
fCReg["BitMapID"].value  := 3023
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 40
fCReg["Designer"].value  := "F"
rCReg.save()

// kmTreeView
rCReg.beginAppend()
fCReg["ClassName"].value := "kmTreeView"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 25
fCReg["BitMapID"].value  := 3041
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 41
fCReg["Designer"].value  := "F"
rCReg.save()

// kmTreeContainer
rCReg.beginAppend()
fCReg["ClassName"].value := "kmTreeContainer"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 25
fCReg["BitMapID"].value  := 3041
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 42
fCReg["Designer"].value  := "F"
rCReg.save()

// kmSeeker
rCReg.beginAppend()
fCReg["ClassName"].value := "kmSeeker"
#if __version__ >= 2 // dBASE Plus or later
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 32
#else
  fCReg["Filename"].value  := cStock
  //fCReg["BitMapID"].value  := 3
  fCReg["BitMapID"].value  := 3012
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 43
fCReg["Designer"].value  := "F"
rCReg.save()

// kmNumSeeker
rCReg.beginAppend()
fCReg["ClassName"].value := "kmNumSeeker"
#if __version__ >= 2 // dBASE Plus or later
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 32
#else
  fCReg["Filename"].value  := cStock
  //fCReg["BitMapID"].value  := 3
  fCReg["BitMapID"].value  := 3012
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 44
fCReg["Designer"].value  := "F"
rCReg.save()

// GridColumnIndexChange.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "GridColumnIndexChange"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 22
fCReg["BitMapID"].value  := 3013
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 45
fCReg["Designer"].value  := "F"
rCReg.save()

/*
    Form Controls found in misc. Other .cc files
*/
// Banner -- found in banner.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "Banner"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 1
fCReg["Designer"].value  := "F"
rCReg.save()

// Clock -- found in Clock.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "ClockField"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1 // set for text image
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 2
fCReg["Designer"].value  := "F"
rCReg.save()

// BooleanComboBox -- found in Combo.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "BooleanComboBox"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 5
fCReg["BitMapID"].value  := 3006
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 3
fCReg["Designer"].value  := "F"
rCReg.save()

// BigCursorEntryfield -- found in Entry.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "BigCursorEntryfield"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 4
fCReg["Designer"].value  := "F"
rCReg.save()

// POSNEG_Entryfield -- found in Entry.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "POSNEG_Entryfield"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 5
fCReg["Designer"].value  := "F"
rCReg.save()

// QDATEENTRYFIELD -- found in Entry.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "QDATEENTRYFIELD"
#if __version__ > 2 // dBASE Plus
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 31 // new image
#else
  fCReg["Filename"].value  := cStock
  //fCReg["BitMapID"].value  := 3
  fCReg["BitMapID"].value  := 3012
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 6
fCReg["Designer"].value  := "F"
rCReg.save()

// ProperEntryField -- found in Entry.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "ProperEntryField"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 7
fCReg["Designer"].value  := "F"
rCReg.save()

// LongYear -- found in Entry.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "LongYear"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 8
fCReg["Designer"].value  := "F"
rCReg.save()

// CCardEntryfield -- found in Entry.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "CCardEntryfield"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 9
fCReg["Designer"].value  := "F"
rCReg.save()

// LanguageEntryfield -- found in Entry.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "LanguageEntryfield"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 10
fCReg["Designer"].value  := "F"
rCReg.save()

// FLASHBUTTON -- found in FlashBtn.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "FLASHBUTTON"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 2
fCReg["BitMapID"].value  := 3024
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 11
fCReg["Designer"].value  := "F"
rCReg.save()

// FontContainer -- found in Font.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "FontContainer"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 19
fCReg["BitMapID"].value  := 3007
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 12
fCReg["Designer"].value  := "F"
rCReg.save()

// jpProgress -- found in jpProgress.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "jpProgress"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 18
fCReg["BitMapID"].value  := 3023
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 13
fCReg["Designer"].value  := "F"
rCReg.save()

// diacrit -- entryfield found in Diacrit2.wfm
rCReg.beginAppend()
fCReg["ClassName"].value := "diacrit"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 14
fCReg["Designer"].value  := "F"
rCReg.save()

// KENCALENDARCONTAINER -- found in KenCal2.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "KENCALENDARCONTAINER"
#if __version__ > 2 // dBASE Plus or later
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 31
#else
  fCReg["Filename"].value  := cStock
  //fCReg["BitMapID"].value  := 19
   fCReg["BitMapID"].value  := 3007
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 14 // new image
fCReg["Designer"].value  := "F"
rCReg.save()

// pKENCALENDARCONTAINER -- found in pKenCal2.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "PKENCALENDARCONTAINER"
#if __version__ > 2 // dBASE Plus or later
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 31
#else
  fCReg["Filename"].value  := cStock
  //fCReg["BitMapID"].value  := 19
   fCReg["BitMapID"].value  := 3007
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 15 // new image
fCReg["Designer"].value  := "F"
rCReg.save()

// KENCALENDAR -- found in KenCal3.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "KENCALENDAR"
#if __version__ > 2 // dBASE Plus or later
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 31
#else
  fCReg["Filename"].value  := cStock
  //fCReg["BitMapID"].value  := 19
  fCReg["BitMapID"].value  := 3007
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 16
fCReg["Designer"].value  := "F"
rCReg.save()

// pKENCALENDAR -- found in pKenCal3.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "PKENCALENDAR"
#if __version__ > 2 // dBASE Plus or later
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 31
#else
  fCReg["Filename"].value  := cStock
  //fCReg["BitMapID"].value  := 19
  fCReg["BitMapID"].value  := 3007
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 17
fCReg["Designer"].value  := "F"
rCReg.save()

// DateTimeSpinbox -- found in DateTimeSpinbox.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "DateTimeSpinbox"
fCReg["Filename"].value  := cStock
fCReg["BitMapID"].value  := 3033
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Form Controls"
fCReg["Order"].value     := 18
fCReg["Designer"].value  := "F"
rCReg.save()

// ProcessingListBox -- found in ListBoxs.cc.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "ProcessingListBox"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 4
fCReg["BitMapID"].value  := 3017
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 19
fCReg["Designer"].value  := "F"
rCReg.save()

// Copy2ClipButton -- found in MoreBtns.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "Copy2ClipButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 2040
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 20
fCReg["Designer"].value  := "F"
rCReg.save()

// TopicInfoButton -- found in MoreBtns.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "TopicInfoButton"
fCReg["Filename"].value  := cResource
fCReg["BitMapID"].value  := 112
fCReg["Width"].value     := RWIDTH
fCReg["Height"].value    := RHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 21
fCReg["Designer"].value  := "F"
rCReg.save()

// kmCallDiacrit -- found in MoreBtns.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "kmCallDiacrit"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 22
fCReg["Designer"].value  := "F"
rCReg.save()

// cursor_on_button -- found in MoreBtns.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "cursor_on_button"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 2
fCReg["BitMapID"].value  := 3024
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 23
fCReg["Designer"].value  := "F"
rCReg.save()

// MOVER2CONTAINER -- found in Mover.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "MOVER2CONTAINER"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 4
fCReg["BitMapID"].value  := 3007
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 24
fCReg["Designer"].value  := "F"
rCReg.save()

// PassVerify -- found in PassVerify.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "PassVerify"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 25
fCReg["Designer"].value  := "F"
rCReg.save()

// PasswordEntry -- found in password.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "PasswordEntry"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 26
fCReg["Designer"].value  := "F"
rCReg.save()

// RowsetSpin -- found in RowsetSpin.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "RowsetSpin"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 9
fCReg["BitMapID"].value  := 3033
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 27
fCReg["Designer"].value  := "F"
rCReg.save()

// SeekerSQL -- found in SeekerSQL.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "SeekerSQL"
#if __version__ >= 2 // dBASE Plus or later
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 32
#else
  fCReg["Filename"].value  := cStock
//  fCReg["BitMapID"].value  := 3
  fCReg["BitMapID"].value  := 3012
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 28
fCReg["Designer"].value  := "F"
rCReg.save()

// STATUSBAR -- found in STATUSBAR.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "STATUSBAR"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 19
fCReg["BitMapID"].value  := 3007
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 29
fCReg["Designer"].value  := "F"
rCReg.save()

// TIMEENTRYFIELD -- found in TimeFiel.cc and/or in TimeFld.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "TIMEENTRYFIELD"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 30
fCReg["Designer"].value  := "F"
rCReg.save()

// zSeeker -- found in zSeeker.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "zSeeker"
#if __version__ >= 2 // dBASE Plus or later
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 32
#else
  fCReg["Filename"].value  := cStock
  //fCReg["BitMapID"].value  := 3
  fCReg["BitMapID"].value  := 3012
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 31
fCReg["Designer"].value  := "F"
rCReg.save()

// PaswordMaskEF -- found in PasswordMaskEF.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "PasswordMaskEF"
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Order"].value     := 26
fCReg["Designer"].value  := "F"
rCReg.save()

// SelPrinterContainer -- found in SelPrinter.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "SelPrinterContainer"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 19
fCReg["BitMapID"].value  := 3007
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 32
fCReg["Designer"].value  := "F"
rCReg.save()

// TEXTFIELDSELECTOR -- found in TEXTFIELDSELECTOR.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "TEXTFIELDSELECTOR"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 8
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 33
fCReg["Designer"].value  := "F"
rCReg.save()

// TEXTFIELDSELECTOR2 -- found in TEXTFIELDSELECTOR.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "TEXTFIELDSELECTOR2"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 8
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 34
fCReg["Designer"].value  := "F"
rCReg.save()

// DragListContainer -- found in DragListContainer.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "DRAGLISTCONTAINER"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 8
fCReg["BitMapID"].value  := 3017
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 35
fCReg["Designer"].value  := "F"
rCReg.save()

// extCombobox -- found in extCombobox.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "EXTCOMBOBOX"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 8
fCReg["BitMapID"].value  := 3006
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 36
fCReg["Designer"].value  := "F"
rCReg.save()

// CopyRowContainer -- found in CopyRow.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "COPYROWCONTAINER"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 8
fCReg["BitMapID"].value  := 3017
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 37
fCReg["Designer"].value  := "F"
rCReg.save()

// FlashImage -- found in FlashImage.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "FLASHIMAGE"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 8
fCReg["BitMapID"].value  := 3015
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 38
fCReg["Designer"].value  := "F"
rCReg.save()

// TapiDialer -- found in TapiDial.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "TAPIDIALER"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 8
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 39
fCReg["Designer"].value  := "F"
rCReg.save()

// HTMLLinkText -- found in HTMLLinkText.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "HTMLLINKTEXT"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 8
fCReg["BitMapID"].value  := 3037 // Text
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 40
fCReg["Designer"].value  := "F"
rCReg.save()

// DateCalendar -- found in DateCalendar.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "DATECALENDAR"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 3
fCReg["BitMapID"].value  := 3012 // Entryfield
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 41
fCReg["Designer"].value  := "F"
rCReg.save()

// PixCalendar -- found in PixCalendar.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "PixCalendar"
#if __version__ > 2 // dBASE Plus or later
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 31
#else
  fCReg["Filename"].value  := cStock
  //fCReg["BitMapID"].value  := 19
  fCReg["BitMapID"].value  := 3007
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 42
fCReg["Designer"].value  := "F"
rCReg.save()

// PixCalendar2 -- found in PixCalendar2.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "PixCalendar2"
#if __version__ > 2 // dBASE Plus or later
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 31
#else
  fCReg["Filename"].value  := cStock
  //fCReg["BitMapID"].value  := 19
  fCReg["BitMapID"].value  := 3007
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 43
fCReg["Designer"].value  := "F"
rCReg.save()

// PixCalendar21 -- found in PixCalendar21.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "PixCalendar21"
#if __version__ > 2 // dBASE Plus or later
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 31
#else
  fCReg["Filename"].value  := cStock
  //fCReg["BitMapID"].value  := 19
  fCReg["BitMapID"].value  := 3007
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 44
fCReg["Designer"].value  := "F"
rCReg.save()

// PixCalendar22 -- found in PixCalendar22.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "PixCalendar22"
#if __version__ > 2 // dBASE Plus or later
  fCReg["Filename"].value  := "creg.dll"
  fCReg["BitMapID"].value  := 31
#else
  fCReg["Filename"].value  := cStock
  //fCReg["BitMapID"].value  := 19
  fCReg["BitMapID"].value  := 3007
#endif
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 45
fCReg["Designer"].value  := "F"
rCReg.save()

// InfoWindowButton -- found in InfoWindowButton.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "InfoWindowButton"
fCReg["Filename"].value  := cStock
fCReg["BitMapID"].value  := 3024
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 46
fCReg["Designer"].value  := "F"
rCReg.save()

// AccordianMainContainer -- found in Accordian.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "AccordianMainContainer"
fCReg["Filename"].value  := cStock
fCReg["BitMapID"].value  := 3007
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 47
fCReg["Designer"].value  := "F"
rCReg.save()

// Accordian -- found in Accordian.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "Accordian"
fCReg["Filename"].value  := cStock
fCReg["BitMapID"].value  := 3007
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 48
fCReg["Designer"].value  := "F"
rCReg.save()

// two new classes (Jan. 2017):
// kmDateWithCalendar -- found in kmDateWithCal.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "kmDateWithCalendar"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 31
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 49
fCReg["Designer"].value  := "F"
rCReg.save()

// kmDTWithCalendar -- found in kmDateWithCal.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "kmDTWithCalendar"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 31
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 50
fCReg["Designer"].value  := "F"
rCReg.save()

// new controls added (March/April, 2020):
// quickdate -- found in quick_date.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "Quick_Date"
fCReg["Filename"].value  := cStock
fCReg["BitMapID"].value  := 3033
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 51
fCReg["Designer"].value  := "F"
rCReg.save()

// Toggle -- found in toggle.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "Toggle"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 9
fCReg["BitMapID"].value  := 3005
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 52
fCReg["Designer"].value  := "F"
rCReg.save()

// Autofill_entryfield -- found in autofill_entryfield.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "autofill_entryfield"
fCReg["Filename"].value  := cStock
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 53
fCReg["Designer"].value  := "F"
rCReg.save()

// Autofill_entryfield2 -- found in autofill_entryfield.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "autofill_entryfield2"
fCReg["Filename"].value  := cStock
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 54
fCReg["Designer"].value  := "F"
rCReg.save()

// Auto_complete -- found in auto_complete.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "Auto_complete"
fCReg["Filename"].value  := cStock
fCReg["BitMapID"].value  := 3012
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Misc. Controls"
fCReg["Order"].value     := 55
fCReg["Designer"].value  := "F"
rCReg.save()

/*
    Report Controls found in RepCntl.cc
*/
// kmTitleText1
rCReg.beginAppend()
fCReg["ClassName"].value := "KMTITLETEXT1"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 1
fCReg["Designer"].value  := "R"
rCReg.save()

// kmTitleText2
rCReg.beginAppend()
fCReg["ClassName"].value := "KMTITLETEXT2"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 2
fCReg["Designer"].value  := "R"
rCReg.save()

// kmStandardText
rCReg.beginAppend()
fCReg["ClassName"].value := "KMSTANDARDTEXT"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 3
fCReg["Designer"].value  := "R"
rCReg.save()

// kmDateText
rCReg.beginAppend()
fCReg["ClassName"].value := "KMDATETEXT"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 4
fCReg["Designer"].value  := "R"
rCReg.save()

// kmFieldHeadingText
rCReg.beginAppend()
fCReg["ClassName"].value := "KMFIELDHEADINGTEXT"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 5
fCReg["Designer"].value  := "R"
rCReg.save()

// kmFieldHeadingBorderText
rCReg.beginAppend()
fCReg["ClassName"].value := "KMFIELDHEADINGBORDERTEXT"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 6
fCReg["Designer"].value  := "R"
rCReg.save()

// kmFieldText
rCReg.beginAppend()
fCReg["ClassName"].value := "KMFIELDTEXT"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 7
fCReg["Designer"].value  := "R"
rCReg.save()

// kmNumFieldText
rCReg.beginAppend()
fCReg["ClassName"].value := "KMNUMFIELDTEXT"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 1
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 8
fCReg["Designer"].value  := "R"
rCReg.save()

// kmCheckBoxText
rCReg.beginAppend()
fCReg["ClassName"].value := "KMCHECKBOXTEXT"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 7
fCReg["BitMapID"].value  := 3005
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 9
fCReg["Designer"].value  := "R"
rCReg.save()

// kmShape
rCReg.beginAppend()
fCReg["ClassName"].value := "KMSHAPE"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 21
fCReg["BitMapID"].value  := 3031
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 10
fCReg["Designer"].value  := "R"
rCReg.save()

// kmRoundedRectangleShape
rCReg.beginAppend()
fCReg["ClassName"].value := "KMROUNDEDRECTANGLESHAPE"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 21
fCReg["BitMapID"].value  := 3031
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 11
fCReg["Designer"].value  := "R"
rCReg.save()

// kmRectangleShape
rCReg.beginAppend()
fCReg["ClassName"].value := "KMRECTANGLESHAPE"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 21
fCReg["BitMapID"].value  := 3031
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 12
fCReg["Designer"].value  := "R"
rCReg.save()

// kmEllipseShape
rCReg.beginAppend()
fCReg["ClassName"].value := "KMELLIPSESHAPE"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 21
fCReg["BitMapID"].value  := 3031
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 13
fCReg["Designer"].value  := "R"
rCReg.save()

// kmCircleShape
rCReg.beginAppend()
fCReg["ClassName"].value := "KMCIRCLESHAPE"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 21
fCReg["BitMapID"].value  := 3031
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 14
fCReg["Designer"].value  := "R"
rCReg.save()

// kmRoundedSquareShape
rCReg.beginAppend()
fCReg["ClassName"].value := "KMROUNDEDSQUARESHAPE"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 21
fCReg["BitMapID"].value  := 3031
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 15
fCReg["Designer"].value  := "R"
rCReg.save()

// kmSquareShape
rCReg.beginAppend()
fCReg["ClassName"].value := "KMSQUARESHAPE"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 21
fCReg["BitMapID"].value  := 3031
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 16
fCReg["Designer"].value  := "R"
rCReg.save()

// kmLine
rCReg.beginAppend()
fCReg["ClassName"].value := "KMLINE"
fCReg["Filename"].value  := cStock
//fCReg["BitMapID"].value  := 11
fCReg["BitMapID"].value  := 3016
fCReg["Width"].value     := SWIDTH
fCReg["Height"].value    := SHEIGHT
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 17
fCReg["Designer"].value  := "R"
rCReg.save()

// pageNumberPlus -- in PageN.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "PAGENUMBERPLUS"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 29
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 18
fCReg["Designer"].value  := "R"
rCReg.save()

// PageXofY -- in PageXofY.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "PAGEXOFY"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 29
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 19
fCReg["Designer"].value  := "R"
rCReg.save()

// BindingPageNumber -- in Binding.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "BINDINGPAGENUMBER"
fCReg["Filename"].value  := "creg.dll"
fCReg["BitMapID"].value  := 29
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 20
fCReg["Designer"].value  := "R"
rCReg.save()

// AltBandColorText -- in AltBandColor.cc
rCReg.beginAppend()
fCReg["ClassName"].value := "ALTBANDCOLORTEXT"
fCReg["Filename"].value  := cStock
fCReg["BitMapID"].value  := 3037
fCReg["Width"].value     := 16
fCReg["Height"].value    := 15
fCReg["Category"].value  := "dUFLP Report Controls"
fCReg["Order"].value     := 21
fCReg["Designer"].value  := "R"
rCReg.save()

/*
    Controls that should *not* appear on the
    component palette:
*/
// TitleText -- found in kenCal2, pKenCal2
rCReg.beginAppend()
fCReg["ClassName"].value := "TITLETEXT"
fCReg["Category"].value  := "dUFLP"
fCReg["Designer"].value  := "N"
rCReg.save()

// DayPushbutton -- found in kenCal2, pKenCal2
rCReg.beginAppend()
fCReg["ClassName"].value := "DAYPUSHBUTTON"
fCReg["Category"].value  := "dUFLP"
fCReg["Designer"].value  := "N"
rCReg.save()

// TitleText -- found in kenCal3, pKenCal3
rCReg.beginAppend()
fCReg["ClassName"].value := "TITLETEXT3"
fCReg["Category"].value  := "dUFLP"
fCReg["Designer"].value  := "N"
rCReg.save()

// DayPushbutton -- found in kenCal3, pKenCal3
rCReg.beginAppend()
fCReg["ClassName"].value := "DAYPUSHBUTTON3"
fCReg["Category"].value  := "dUFLP"
fCReg["Designer"].value  := "N"
rCReg.save()




// done
qCReg.active := false
release object qCReg

// Let user know:
if not bCalled
   msgbox( "The next time you open a form or report in the designer "+;
           "that uses one of the visual control files in the dUFLP, or if "+;
           "creating a new form/report and you have opened at least one "+;
           "of these files, the Component Palette should have new tabs "+;
           "with these components grouped ...", "Done", 64 )
endif

/*
   End of program: ComponentPaletteUpdate.prg
*/
