// example for the use of the GeoTools.cc custom class

// !!! WARNING !!!

/* This CLASS works only if the timezone passed to the setdate method is in 
	English! In my eyes, this is a bug in the windows operating system. 
	All the timezone information is stored in the registry, but as a locale
	string. Problem here is that in some cases, different time zones have the 
	same locale designation F.i. on my german system, 
	
	Cen. Australia Standard Time  and
	Aus Central W. Standard Time
	
	have the same locale designation : Zentralaustralische Normalzeit
	
	only in english, all the timezones can be identified without ambiguity
*/
clear
?
set procedure to :dUFLP:geotools.cc

tzo = new geotools()

// the following must be set if you want to use the isnight property
// if you don't use it, the isnight will reflect the situation 
// at 0 latitude and 0 longitude for the time given in setdate

//Brisbane: tzo.setgeoloc(-27.470125,153.021072)
// Brussels: tzo.setgeoloc(50.8505,4.3488)
//Adelaide tzo.setgeoloc(-34.9285,138.6007)
//Sydney tzo.setgeoloc(-33.8688,151.2093)
// New York tzo.setgeoloc(40.7128,-74.0060)
tzo.setgeoloc(37.9101,-122.0652)
if tzo.error.code<>0
	?"an error occurred"
	?"tzo.error= "+tzo.error.message
endif// Gaetano's values

// The following method can be used without the need to reinitiate the 
// timezone-object if you want to use other params. 
// Parameters must be of the form:

// "YYYY-MM-DD hh:mm:ss"    -> SQL timestamp format, the ss can be omitted
// As said above, timezone has to be in English 

//,"AUS Eastern Standard Time"
tzo.setdate("2021-02-24 10:10:00","Pacific Standard Time")  // remove the TZ name to test the default
if tzo.error.code<>0 // add error handling code here
	?"an error occurred"
	?"tzo.error= "+tzo.error.message
endif

// after that, all the properties are set taking the parameters into account
// and DLST

? "Time zone offset to GMT for Walnut Creek (CA) - "+tzo.tmzkey+":  "+ tzo.offset /60+ " hours"
? "Sunrise at lat. "+tzo.lat+" and long. "+ tzo.long+" is at: "+tzo.sunrise
? "Sunset is at  " + tzo.sunset
? "The specified time ("+tzo.tmstmp+")falls into the night : " + tzo.isnight
? "HasDST: "+tzo.hasDST
?"isDST: "+tzo.isDST
?
?"--- coordsToDistance demo - Brisbane to Sydney ---"
lat1=-27.4698
lat2=-33.8814
long1=153.0251
long2=151.1707
?
?"----- in km"
?tzo.coordsToDistance(lat1,lat2,long1,long2) //default is metric, ad "false" for Miles
?
?"----- in miles"
? tzo.coordsToDistance(lat1,lat2,long1,long2,false)
?
tzo.setgeoloc(-33.8688,151.2093)
if tzo.error.code<>0
	?"an error occurred"
	?"tzo.error= "+tzo.error.message
else
	
endif
tzo = null
close proc :dUFLP:geotools.cc


