// Metric: 6 - Pixels only
/*
   --------------------------------------------------------------------
   jpProgress.cc
   Programmers..: Jonny Kwekkeboom
                  Jean-Pierre Martel
                  Ken Mayer
                  With thanks to Gary White and Marko Mihorko
   Date.........: February, 2002
   Rev. History.: March 27, 2002 -- Robert Cline --
                       Eliminate Some Screen Flickering
                       Changed Approaches to some things
                       Allowed for backwards manipulation of bar
                  April 1, 2002 -- Jean-Pierre Martel -- 
                       Progress_units defined once in 
                       Progress_onOpen() rather than in 
                       UpdateProgress()
                  April 24, 2002 -- Fixed some things that JP did
                       in his code that wiped out the user's
                       color selections.
                  June 24, 2002 -- KJM -- dealt with possibility
                       that rangeMax could get set to zero,
                       causing a divide-by-zero error/blue-screen/crash
                       of the software ...

   Description..: This is a custom progress bar that allows 
                  the developer to set the color(s),
                  is a "solid bar", rather than the dB2K
                  component which is a series of boxes,
                  and has text ON the progress bar that
                  changes color as the bar progresses, showing
                  the percentage.
   Usage........: Place in your Component Palette
                     set procedure to :dUFLP:jpProgress.cc additive
                  Place the component on your form, and move
                  to where you need it to be.

                  NOTE: When you place this on the form, you will
                  see just the container, with no border ...
                  All of that is taken care of in the container's
                  onOpen event.

                  In the form's Open method or onOpen event 
                  place the code to set the colors. Note that 
                  if you do this in the form's over-written 
                  Open event that the colors are drawn before
                  the form is opened, which may look better
                  to your user:

                  function form_Open
                     // color of the background
                     form.jpProgress1.ProgressColorBackground = "silver"
                     // color of the progress bar itself:
                     form.jpProgress1.ProgressColorForeground = "red"
                     // color of the text before the progress bar
                     // covers it:
                     form.jpProgress1.ProgressTextBackColor = "black"
                     // color of the text AFTER the progress bar
                     // covers it:
                     form.jpProgress1.ProgressTextHighColor = "white"
                     // make sure we "re-initialize" the control or
                     // the new color settings won't "take":
                     form.jpProgress1.onOpen()
                   return super::Open() // don't forget this!

                  // Set the rangeMin property if not starting
                  // at 1:
                  form.jpProgress1.rangeMin := 50

                  // Set the rangeMax property of the jpProgress:
                  form.jpProgress1.rangeMax := 250

                  // To actually increment the progress bar, call the
                  // UpdateProgress method with a value that is 
                  // in the range between 1 and the max value given
                  // above. The progress bar's UpdateProgress method
                  // will calculate the percentage processed ...:
                  
                  form.jpProgress1.UpdateProgress( 50 ) 
                                // 50 = some value being processed


   Example......: see TestJPProgress.wfm -- it is set up to show
                  this control "in action", and you can tinker
                  with the colors (in the source code) and test it ...
                  the form has a standard dB2K progress object
                  on it for comparison ...

   Side Effects.: This control sets the form's metric to pixels,
                  because if it doesn't, the calculations cause
                  odd behaviors. The container object does not
                  have a metric property itself, so the form's
                  metric gets changed.
   --------------------------------------------------------------------
*/

class jpProgress(parentObj) of CONTAINER(parentObj) custom
   with (this)
      onOpen = class::PROGRESS_ONOPEN
      onDesignOpen = {;this.borderStyle = 4 }
      height = 20
      width = 50
      parent.metric := 6  // Pixels
      metric = 6	// Pixels
   endwith

   // custom properties, with default properties:
   // set these here -- if they're in the Initialize
   // method, they override whatever the user may
   // have set.
   this.ProgressColorBackground = "BtnFace"
   this.ProgressColorForeground = "0xaa0000" // dark blue
   this.ProgressTextBackColor   = "black"
   this.ProgressTextHighColor   = "white"
   this.rangeMin                = 0   // assume we start at 0
   this.rangeMax                = 100 // some default
   this.LastVal                 = -1  // Force Update First Time

   this.R_OUTSIDE = new RECTANGLE(this)
   with (this.R_OUTSIDE)
      left = 0
      top = -300
      width = this.width
      height = this.height
      text = ""
      border = false
      borderStyle = 2	// Lowered
      metric = 6	// Pixels
   endwith

   this.R_INSIDE = new RECTANGLE(this)
   with (this.R_INSIDE)
      left = 2
      top = -300
      width = 0
      height = this.height - 5
      text = ""
      border = false
      borderStyle = 3	// None
      metric = 6	// Pixels
   endwith

   this.TEXT1A = new TEXT(this)
   with (this.TEXT1A)
      height = this.height - 5
      left = floor(this.width/2) - 17
      top = -300
      width = 5
      alignVertical = 1	// Middle
      alignHorizontal = 2	// Right
      picture = "9"
      fontSize = 8
      fontBold = true
      text = "1"
      metric = 6	// Pixels
   endwith

   this.TEXT1B = new TEXT(this)
   with (this.TEXT1B)
      height = this.height - 5
      left = floor(this.width/2) - 10
      top = -300
      width = 5
      alignVertical = 1	// Middle
      alignHorizontal = 2	// Right
      picture = "9"
      fontSize = 8
      fontBold = true
      text = "0"
      metric = 6	// Pixels
   endwith

   this.TEXT1C = new TEXT(this)
   with (this.TEXT1C)
      height = this.height - 5
      left = floor(this.width/2) - 3
      top = -300
      width = 5
      alignVertical = 1	// Middle
      alignHorizontal = 2	// Right
      picture = "9"
      fontSize = 8
      fontBold = true
      text = "0"
      metric = 6	// Pixels
   endwith

   this.TEXT1D = new TEXT(this)
   with (this.TEXT1D)
      height = this.height - 5
      left = floor(this.width/2) + 8
      top = -300
      width = 8
      alignVertical = 1	// Middle
      alignHorizontal = 2	// Right
      picture = "X"
      fontSize = 8
      fontBold = true
      text = "%"
      metric = 6	// Pixels
   endwith

   class::Initialize()

   function Initialize
      // set Colors:
      this.R_OUTSIDE.colorNormal := this.ProgressTextBackColor+"/"+;
                                    this.ProgressColorBackground
      this.R_INSIDE.colorNormal  := this.ProgressTextHighColor+"/"+;
                                    this.ProgressColorForeground
      this.TEXT1A.colorNormal    := this.ProgressTextBackColor+"/"+;
                                    this.ProgressColorBackground
      this.TEXT1B.colorNormal    := this.ProgressTextBackColor+"/"+;
                                    this.ProgressColorBackground
      this.TEXT1C.colorNormal    := this.ProgressTextBackColor+"/"+;
                                    this.ProgressColorBackground
      this.TEXT1D.colorNormal    := this.ProgressTextBackColor+"/"+;
                                    this.ProgressColorBackground
   return

   function UpdateProgress( nValue )
      // Update progress Bar -- move it, and change color of
      // percentage text ...
      local nCurVal, cNr, nWidth

      // deal with user trying to pass a value
      // outside of the range ...
      nValue = max(min(nValue, this.rangeMax), this.RangeMin)
     

      // Determine percentage based on value passed
      // to this routine and the rangeMin and RangeMax properties ...
      if (this.rangeMax-this.rangeMin) > 0 and (this.rangeMax > 0)
         // it's possible to try to divide by zero here,
         // if rangeMax and rangeMin are the same ...
         nCurVal = int( ( nValue / (this.rangeMax-this.rangeMin) ) * 100 )
      else
         nCurVal = 100 // make it 100% ...
      endif

      // we're not dealing with a value > 100
      if nCurVal > 100
         nCurVal := 100
      endif

      // If Calculate percent has changed let's do things
      if (nCurVal == this.LastVal)
         return
      else
         cNr = str(nCurVal,3,0)

         // set text properties:
         this.Text1A.text = cNr.charat(0)
         this.Text1B.text = cNr.charat(1)
         this.Text1C.text = cNr.charat(2)

         // Calculate width of progress
         nWidth = floor(this.Progress_units*nCurVal)

         // Only deal with the colors of the text if the bar is going to 
         // trip past the left side of the text
         if (this.R_INSIDE.WIDTH <= this.Text1A.Left and this.Text1A.Left < nWidth) or ;
            (this.R_INSIDE.WIDTH >= this.Text1A.Left and this.Text1a.Left > nWidth)
            // If bar is going to overlay text set highlight color
            if (nWidth > this.Text1A.left)
              this.Text1A.colornormal := this.ProgressTextHighColor+"/"+;
                                         this.ProgressColorForeground
            // Bar is going to unoverlay text go back to default color
            else
              this.TEXT1A.colorNormal := this.ProgressTextBackColor+"/"+;
                                         this.ProgressColorBackground
            endif
         endif

         // Only deal with the colors of the text if the bar is going to 
         // trip past the left side of the text
         if (this.R_INSIDE.WIDTH <= this.Text1B.Left and this.Text1B.Left < nWidth) or ;
            (this.R_INSIDE.WIDTH >= this.Text1B.Left and this.Text1B.Left > nWidth)
            // If bar is going to overlay text set highlight color
            if (nWidth > this.Text1B.left)
              this.Text1B.colornormal := this.ProgressTextHighColor+"/"+;
                                         this.ProgressColorForeground
            // Bar is going to unoverlay text go back to default color
            else
              this.TEXT1B.colorNormal := this.ProgressTextBackColor+"/"+;
                                         this.ProgressColorBackground
            endif
         endif

         // Only deal with the colors of the text if the bar is going to 
         // trip past the left side of the text
         if (this.R_INSIDE.WIDTH <= this.Text1C.Left and this.Text1C.Left < nWidth) or ;
            (this.R_INSIDE.WIDTH >= this.Text1C.Left and this.Text1C.Left > nWidth)
            // If bar is going to overlay text set highlight color
            if (nWidth > this.Text1C.left)
              this.Text1C.colornormal := this.ProgressTextHighColor+"/"+;
                                         this.ProgressColorForeground
            // Bar is going to unoverlay text go back to default color
            else
              this.TEXT1C.colorNormal := this.ProgressTextBackColor+"/"+;
                                         this.ProgressColorBackground
            endif
         endif

         // Only deal with the colors of the text if the bar is going to 
         // trip past the left side of the text
         if (this.R_INSIDE.WIDTH <= this.Text1D.Left and this.Text1D.Left < nWidth) or ;
            (this.R_INSIDE.WIDTH >= this.Text1D.Left and this.Text1D.Left > nWidth)
            // If bar is going to overlay text set highlight color
            if (nWidth > this.Text1D.left)
              this.Text1D.colornormal := this.ProgressTextHighColor+"/"+;
                                         this.ProgressColorForeground
            // Bar is going to unoverlay text go back to default color
            else
              this.TEXT1D.colorNormal := this.ProgressTextBackColor+"/"+;
                                         this.ProgressColorBackground
            endif
         endif
         this.R_INSIDE.width = nWidth   // Set width of progress bar
         this.lastval = nCurVal         // Set last value to cur calc value
      endif  // Change in Value
   return

   Function Progress_onOpen
      // Set height/width/positions:
      this.borderStyle = 3	// None
      this.R_Outside.top    := 0
      this.R_Outside.height := this.height
      this.R_Outside.width  := this.width
      this.R_Inside.top     := 2
      this.R_Inside.height  := this.height - 5
      this.Text1A.left   := floor(this.width/2) - 17
      this.Text1A.top    := 2
      this.Text1A.height := this.height - 5
      this.Text1B.left   := floor(this.width/2) - 10
      this.Text1B.top    := 2
      this.Text1B.height := this.height - 5
      this.Text1C.left   := floor(this.width/2) - 3
      this.Text1C.top    := 2
      this.Text1C.height := this.height - 5
      this.Text1D.left   := floor(this.width/2) + 8
      this.Text1D.top    := 2
      this.Text1D.height := this.height - 5
      this.Progress_units = (this.R_OUTSIDE.width - 5)/100
      this.initialize()      
   return

endclass

