/*
   KENCAL4.CC
   Author  : Ken Mayer
   Original: August, 2000
   Updated : March, 2004

   NOTE: This is a special version of the calendar class
       :dUFLP:kenCal3.cc

   This one relies on a different version of the LoadHolidays
   method -- one found in a different form. It is modified in such
   a way that it can and should only be used in the kmScheduler
   set of programs found in kmScheduler folderin the dUFLP.

   Do NOT use this in place of :dUFLP:kenCal3.cc in the dUFLP,
   as the LoadHolidays method here is completely different and
   will not function properly.
*/

/*
   -------------------------------------------------------
   The container class here is what does all the work ...
   There are some custom button/text control definitions 
   after the container definition.
   -------------------------------------------------------
*/
class KENCALENDAR(parentObj) of CONTAINER(parentObj) custom
   set procedure to :dUFLP:dateex.cc additive
   set procedure to :dUFLP:holiday.cc additive

   with (this)
      width        := 25
      height       := 10.7273
      metric       := 0	// Chars
      transparent  := true
      onOpen       := class::CONTAINEROPEN
      onDesignOpen := class::CheckMetric
   endwith

   #define TITLEROW 2.3182
   #define ROW1     3.3182
   #define ROW2     4.3182
   #define ROW3     5.3182
   #define ROW4     6.3182
   #define ROW5     7.3182
   #define ROW6     8.3182
   #define COL1     1
   #define COL2     4.1429
   #define COL3     7.2857
   #define COL4     10.4286
   #define COL5     13.5714
   #define COL6     16.7143
   #define COL7     19.8571

   /*
      Color definitions are custom properties of the container
      used to define the buttons:
        NormalColor is the normal color used if not selected
        SelectedColor is what it sounds like, so that the
           currently selected date stands out
        SundayColor is used to show the first column
           (Sunday) ... a lot of calendars do this, so 
           I figured I would, too ...
        HolidayColor is used to show holidays ...

           Note that holidays that are in the array defined
           by "LoadHolidays" (a method of the container class)
           will appear in this color as well, unless they 
           are the "selected" item ...
   */
   this.NORMALCOLOR   = "BLACK/BTNFACE"
   this.SELECTEDCOLOR = "RED/WHITE"
   this.SUNDAYCOLOR   = "MAROON/BTNFACE"
   this.HOLIDAYCOLOR  = "WHITE/NAVY"

   // instance of dateex class and holiday class:
   this.DateEx        = new DateEx()
   this.Holiday       = new Holiday()
   // used to set speedtips ...
   this.cSpeedTip     = ""
   // currentDate and beginDate:
   this.currentDate   = {}
   this.beginDate     = {}


   this.YEARMINUS10 = new PUSHBUTTON(this)
   with (this.YEARMINUS10)
      onClick = class::YEARMINUS10_ONCLICK
      height = 0.95
      left = 1.57
      top = 0.18
      width = 3
      text = "10"
      fontSize = 8
      speedBar = true
      speedTip = "10 Years Back"
      colorNormal = "red/BtnFace"
   endwith

   this.PREVYEARPUSHBUTTON = new PUSHBUTTON(this)
   with (this.PREVYEARPUSHBUTTON)
      onClick  := CLASS::PREVYEARPUSHBUTTON_OnClick
      upBitmap := "RESOURCE #852"
      text     := ""
      height   := 0.95
      width    := 2.6
      left     := 4.57
      top      := 0.18
      speedBar := true
      speedTip := "Previous Year"
   endwith

   this.TEXTYEAR = new TEXT(this)
   with (this.TEXTYEAR)
      fontSize        := 10
      text            := "9999"
      alignHorizontal := 1 // center
      alignVertical   := 1 // middle
      height          := 0.95
      left            := 7.00
      top             := 0.18
      width           := 10.43
      border          := true
      borderStyle     := 1
   endwith

   this.NEXTYEARPUSHBUTTON = new PUSHBUTTON(this)
   with (this.NEXTYEARPUSHBUTTON)
      onClick  := CLASS::NEXTYEARPUSHBUTTON_OnClick
      upBitmap := "RESOURCE #855"
      text     := ""
      height   := 0.95
      width    := 2.6
      left     := 17.4286
      top      := 0.18
      speedBar := true
      speedTip := "Next Year"
   endwith

   this.YEARPLUS10 = new PUSHBUTTON(this)
   with (this.YEARPLUS10)
      onClick = class::YEARPLUS10_ONCLICK
      height = 0.95
      left = 20.1429
      top = 0.18
      width = 3
      text = "10"
      fontSize = 8
      speedBar = true
      speedTip = "10 Years Ahead"
      colorNormal = "green/BtnFace"
   endwith

   this.MONTHMINUS3 = new PUSHBUTTON(this)
   with (this.MONTHMINUS3)
      onClick = class::MONTHMINUS3_ONCLICK
      height = 0.95
      left = 1.57
      top = 1.14
      width = 3
      text = "3"
      fontSize = 8
      speedBar = true
      speedTip = "3 Months Back"
      colorNormal = "red/BtnFace"
   endwith

   this.PREVMONTHPUSHBUTTON = new PUSHBUTTON(this)
   with (this.PREVMONTHPUSHBUTTON)
      onClick  := CLASS::PREVMONTHPUSHBUTTON_OnClick
      upBitmap := "RESOURCE #852"
      text     := ""
      height   := 0.9545
      width    := 2.6
      left     := 4.57
      top      := 1.14
      speedBar := true
      speedTip := "Previous Month"
   endwith

   this.TEXTMONTH = new TEXT(this)
   with (this.TEXTMONTH)
      fontSize        := 10
      text            := "Month"
      alignHorizontal := 1 // center
      alignVertical   := 1 // middle
      height          := 0.95
      left            := 7.00
      width           := 10.43
      top             := 1.14
      border          := true
      borderStyle     := 1
   endwith

   this.NEXTMONTHPUSHBUTTON = new PUSHBUTTON(this)
   with (this.NEXTMONTHPUSHBUTTON)
      onClick  := CLASS::NEXTMONTHPUSHBUTTON_OnClick
      height   := 0.95
      width    := 2.6
      left     := 17.43
      top      := 1.14
      upBitmap := "RESOURCE #855"
      text     := ""
      speedBar := true
      speedTip := "Next Month"
   endwith

   this.MONTHPLUS3 = new PUSHBUTTON(this)
   with (this.MONTHPLUS3)
      onClick = class::MONTHPLUS3_ONCLICK
      height = 0.95
      left = 20.1429
      top = 1.14
      width = 3
      text = "3"
      fontSize = 8
      speedBar = true
      speedTip = "3 Months Ahead"
      colorNormal = "green/BtnFace"
   endwith

   this.TITLESUNDAY = new TitleText3(this)
   with (this.TITLESUNDAY)
      left := COL1
      top  := TITLEROW
   endwith

   this.TITLEMONDAY = new TitleText3(this)
   with (this.TITLEMONDAY)
      left := COL2
      top  := TITLEROW
      text := "M"
   endwith

   this.TITLETUESDAY = new TitleText3(this)
   with (this.TITLETUESDAY)
      left := COL3
      top  := TITLEROW
      text := "T"
   endwith


   this.TITLEWEDNESDAY = new TitleText3(this)
   with (this.TITLEWEDNESDAY)
      left := COL4
      top  := TITLEROW
      text := "W"
   endwith


   this.TITLETHURSDAY = new TitleText3(this)
   with (this.TITLETHURSDAY)
      left := COL5
      top  := TITLEROW
      text := "T"
   endwith


   this.TITLEFRIDAY = new TitleText3(this)
   with (this.TITLEFRIDAY)
      left := COL6
      top  := TITLEROW
      text := "F"
   endwith


   this.TITLESATURDAY = new TitleText3(this)
   with (this.TITLESATURDAY)
      left := COL7
      top  := TITLEROW
   endwith

   this.ROW1COL1 = new DayPushButton3(this)
   with (this.ROW1COL1)
      left = COL1
      top = ROW1
   endwith

   this.ROW1COL2 = new DayPushButton3(this)
   with (this.ROW1COL2)
      left = COL2
      top = ROW1
   endwith

   this.ROW1COL3 = new DayPushButton3(this)
   with (this.ROW1COL3)
      left = COL3
      top = ROW1
   endwith

   this.ROW1COL4 = new DayPushButton3(this)
   with (this.ROW1COL4)
      left = COL4
      top = ROW1
   endwith

   this.ROW1COL5 = new DayPushButton3(this)
   with (this.ROW1COL5)
      left = COL5
      top = ROW1
   endwith

   this.ROW1COL6 = new DayPushButton3(this)
   with (this.ROW1COL6)
      left = COL6
      top = ROW1
   endwith

   this.ROW1COL7 = new DayPushButton3(this)
   with (this.ROW1COL7)
      left = COL7
      top = ROW1
   endwith

   this.ROW2COL1 = new DayPushButton3(this)
   with (this.ROW2COL1)
      left = COL1
      top = ROW2
   endwith

   this.ROW2COL2 = new DayPushButton3(this)
   with (this.ROW2COL2)
      left = COL2
      top = ROW2
   endwith

   this.ROW2COL3 = new DayPushButton3(this)
   with (this.ROW2COL3)
      left = COL3
      top = ROW2
   endwith

   this.ROW2COL4 = new DayPushButton3(this)
   with (this.ROW2COL4)
      left = COL4
      top = ROW2
   endwith

   this.ROW2COL5 = new DayPushButton3(this)
   with (this.ROW2COL5)
      left = COL5
      top = ROW2
   endwith

   this.ROW2COL6 = new DayPushButton3(this)
   with (this.ROW2COL6)
      left = COL6
      top = ROW2
   endwith

   this.ROW2COL7 = new DayPushButton3(this)
   with (this.ROW2COL7)
      left = COL7
      top = ROW2
   endwith

   this.ROW3COL1 = new DayPushButton3(this)
   with (this.ROW3COL1)
      left = COL1
      top = ROW3
   endwith

   this.ROW3COL2 = new DayPushButton3(this)
   with (this.ROW3COL2)
      left = COL2
      top = ROW3
   endwith

   this.ROW3COL3 = new DayPushButton3(this)
   with (this.ROW3COL3)
      left = COL3
      top = ROW3
   endwith

   this.ROW3COL4 = new DayPushButton3(this)
   with (this.ROW3COL4)
      left = COL4
      top = ROW3
   endwith

   this.ROW3COL5 = new DayPushButton3(this)
   with (this.ROW3COL5)
      left = COL5
      top = ROW3
   endwith

   this.ROW3COL6 = new DayPushButton3(this)
   with (this.ROW3COL6)
      left = COL6
      top = ROW3
   endwith

   this.ROW3COL7 = new DayPushButton3(this)
   with (this.ROW3COL7)
      left = COL7
      top = ROW3
   endwith

   this.ROW4COL1 = new DayPushButton3(this)
   with (this.ROW4COL1)
      left = COL1
      top = ROW4
   endwith

   this.ROW4COL2 = new DayPushButton3(this)
   with (this.ROW4COL2)
      left = COL2
      top = ROW4
   endwith

   this.ROW4COL3 = new DayPushButton3(this)
   with (this.ROW4COL3)
      left = COL3
      top = ROW4
   endwith

   this.ROW4COL4 = new DayPushButton3(this)
   with (this.ROW4COL4)
      left = COL4
      top = ROW4
   endwith

   this.ROW4COL5 = new DayPushButton3(this)
   with (this.ROW4COL5)
      left = COL5
      top = ROW4
   endwith

   this.ROW4COL6 = new DayPushButton3(this)
   with (this.ROW4COL6)
      left = COL6
      top = ROW4
   endwith

   this.ROW4COL7 = new DayPushButton3(this)
   with (this.ROW4COL7)
      left = COL7
      top = ROW4
   endwith

   this.ROW5COL1 = new DayPushButton3(this)
   with (this.ROW5COL1)
      left = COL1
      top = ROW5
   endwith

   this.ROW5COL2 = new DayPushButton3(this)
   with (this.ROW5COL2)
      left = COL2
      top = ROW5
   endwith

   this.ROW5COL3 = new DayPushButton3(this)
   with (this.ROW5COL3)
      left = COL3
      top = ROW5
   endwith

   this.ROW5COL4 = new DayPushButton3(this)
   with (this.ROW5COL4)
      left = COL4
      top = ROW5
   endwith

   this.ROW5COL5 = new DayPushButton3(this)
   with (this.ROW5COL5)
      left = COL5
      top = ROW5
   endwith

   this.ROW5COL6 = new DayPushButton3(this)
   with (this.ROW5COL6)
      left = COL6
      top = ROW5
   endwith

   this.ROW5COL7 = new DayPushButton3(this)
   with (this.ROW5COL7)
      left = COL7
      top = ROW5
   endwith

   this.ROW6COL1 = new DayPushButton3(this)
   with (this.ROW6COL1)
      left = COL1
      top = ROW6
   endwith

   this.ROW6COL2 = new DayPushButton3(this)
   with (this.ROW6COL2)
      left = COl2
      top = ROW6
   endwith

   this.ROW6COL3 = new DayPushButton3(this)
   with (this.ROW6COL3)
      left = COL3
      top = ROW6
   endwith

   this.ROW6COL4 = new DayPushButton3(this)
   with (this.ROW6COL4)
      left = COL4
      top = ROW6
   endwith

   this.ROW6COL5 = new DayPushButton3(this)
   with (this.ROW6COL5)
      left = COL5
      top = ROW6
   endwith

   this.ROW6COL6 = new DayPushButton3(this)
   with (this.ROW6COL6)
      left = COL6
      top = ROW6
   endwith

   this.ROW6COL7 = new DayPushButton3(this)
   with (this.ROW6COL7)
      left = COL7
      top = ROW6
   endwith

   this.TodayPushButton = new PUSHBUTTON(this)
   with (this.TodayPushButton)
      onClick  = CLASS::TodayPushButton_OnClick
      height   = 0.82
      left     = 8.29
      top      = 9.45
      width    = 10.00
      text     = "Today"
      upBitmap = "RESOURCE #855"
      fontSize = 10
      speedTip = "Go to Today's Date"
   endwith

   function CheckMetric
      // called from container's onDesignOpen:
      if form.metric # 0
         msgbox( "Form's metric property is not '0 - Chars', "+;
                 "this will not display properly.", ;
                 "Bad Form Metric!", 16 )
      endif

   function containerOpen
      // default date if needed:
      if type( 'this.currentDate' ) # "D" or ; 
         empty( this.currentDate )
         this.currentDate = date()
      endif

      // Set color of "sunday" title button
      this.titleSunday.colorNormal := this.SUNDAYCOLOR

      // store starting date value, in
      // case there's a cancel option,
      // the user can return to the 'beginDate':
      this.beginDate = this.currentDate

      // newMonth routine
      class::NewMonth()

   function newMonth
      // this function deals with setting values based
      // on changes to month/year -- determine where 
      // first day is, last day is, everything in 
      // between: set button texts, colors ...

      // First: need to determine what level we're at --
      // are we working from container, or control
      // on container (cRef needs to point to the container):
      if "CONTAINER" $ upper( this.baseClassName )
         // we're on the container
         oRef = this  
      else
         // we're on a pushbutton _on_ the container
         oRef = this.parent 
      endif

      // Make sure holiday list is current ...
      class::LoadHolidays()

      // now to get first day of month and last day of
      // month from 'currentDate'
      local dDate, dFirst, dLast, nDay, nStart, nRows, nColumns
      dDate  = oRef.currentDate
      dFirst = oRef.dateEx.fdom( dDate )
      dLast  = oRef.dateEx.ldom( dDate )
      nDay   = day( dDate ) // current day of month

      // character values for month and year:
      oRef.textMonth.text := oRef.dateEx.intlCMon( dDate )
      oRef.textYear.text  := ltrim( str( year( dDate ) ) )

      // now the fun part ... assigning values, based
      // on where we start:
      nStart = dow( oRef.currentDate )

      // Clear out any outstanding speedTips
      for nRows = 1 to 6
          for nColumns = 1 to 7
              cObject = "oRef.Row1Col"+nColumns
              cSpeedTip = cObject+".speedTip"
              &cSpeedTip. := ""
          next
      next

      // We need to loop through everything, and set
      // visible properties as well as text if it's going
      // to display:
      nCurrent = 0
      for nRows = 1 to 6
          for nColumns = 1 to 7
              nCurrent++
              // start at day 1, but ...
              // for first row, loop through until we find
              // the first one (and make everything before
              // the first day of month invisible):
              if nCurrent == 1
                 do while nCurrent <= nColumns
                    if dow( dFirst ) == nColumns
                       // set the text:
                       cObject = "oRef.Row1Col"+nColumns
                       cText   = cObject+".text"
                       &cText. := "1"
                       // Set colors:
                       cColor = cObject+".colorNormal"
                       // if the day we're setting is the
                       // selected/current date:
                       dTestDate = new date(year(dDate), ;
                                   month(dDate)-1, nCurrent)
                       // Make it a real date
                       dTestDate = ctod( left( dTestDate.toLocaleString(), ;
                                   iif( set("CENTURY") == "ON", 10, 8 ) ) )
                       if nDay == nCurrent
                          &cColor. := oRef.SELECTEDCOLOR
                          oRef.currentButton = &cObject.
                          cSpeedTip = cObject+".speedTip"
                          if class::isHoliday( dtestDate )
                             &cSpeedTip. = oRef.cSpeedTip
                          endif                       
                       else
                          // is it a sunday or a holiday?
                          cSpeedTip = cObject+".speedTip"
                          if nColumns == 1 
                             &cColor. := oRef.SUNDAYCOLOR
                             if class::IsHoliday( dTestDate )
                                &cColor.    := oRef.HOLIDAYCOLOR
                                &cSpeedTip. := oRef.cSpeedTip
                             endif
                          elseif class::IsHoliday( dTestDate )
                             &cColor. := oRef.HOLIDAYCOLOR
                             &cSpeedTip. := oRef.cSpeedTip
                          else
                             // it's any other day of week ...
                             &cColor. := oRef.NORMALCOLOR
                          endif
                       endif // nDay == nCurrent

                       // set visible property
                       cVisible = cObject+".visible"
                       &cVisible. := true
                       // finish the row
                       for i = nColumns to 7
                           cObject = "oRef.Row1Col"+i
                           cText   = cObject+".text"
                           &cText. := "1"
                           // set visible property
                           cVisible = cObject+".visible"
                           &cVisible. := true
                       next
                       exit // out of this loop, we're done
                    else 
                       if nColumns > 7
                          nColumns = 7
                          exit
                       endif
                       cObject = "oRef.Row1Col"+nColumns
                       cVisible = cObject+".visible"
                       &cVisible. := false
                    endif // dow( dFirst ) = nColumns
                    nColumns++ // increment column counter
                    loop       // top of _this_ loop
                 enddo // while nCurrent <= nColumns
                 loop // back to columns for/next loop
              endif // nCurrent = 1

              // now to attempt to handle setting
              // text and colors ...
              cObject = "oRef.Row"+nRows+"Col"+nColumns
              cSpeedTip = cObject+".speedTip"
              &cSpeedTip := "" // clear out whatever's there
              // if we're inside the month's dates:
              if nCurrent > 1 and nCurrent <= day( dLast )
                 // set the text property:
                 cText = cObject+".text"
                 &cText. := ""+nCurrent
                 // set the button to visible:
                 cVisible = cObject+".visible"
                 &cVisible. := true
                 // deal with colors:
                 cColor = cObject+".colorNormal"
                 // if the day we're setting is the
                 // selected/current date:
                 dTestDate = new date(year(dDate), ;
                             month(dDate)-1, nCurrent)
                 // Make it a real date
                 dTestDate = ctod( left( dTestDate.toLocaleString(), ;
                             iif( set("CENTURY") == "ON", 10, 8 ) ) )
                 if nDay == nCurrent
                    &cColor. := oRef.SELECTEDCOLOR
                    oRef.currentButton = &cObject.
                    cSpeedTip = cObject+".speedTip"
                    if class::isHoliday( dtestDate )
                       &cSpeedTip. = oRef.cSpeedTip
                    endif                       
                 else
                    // is it a sunday or a holiday?
                    cSpeedTip = cObject+".speedTip"
                    if nColumns == 1 
                       &cColor. := oRef.SUNDAYCOLOR
                       if class::IsHoliday( dTestDate )
                          &cColor.    := oRef.HOLIDAYCOLOR
                          &cSpeedTip. := oRef.cSpeedTip
                       endif
                    elseif class::IsHoliday( dTestDate )
                       &cColor. := oRef.HOLIDAYCOLOR
                       &cSpeedTip. := oRef.cSpeedTip
                    else
                       // it's any other day of week ...
                       &cColor. := oRef.NORMALCOLOR
                    endif
                 endif // nDay == nCurrent
              else
                 // we want to make it invisible
                 cVisible = cObject+".visible"
                 &cVisible. := false
              endif // nCurrent > 1 and nCurrent <= day( dLast )
          next // nColumns
       next // nRows

   function TodayPushButton_onClick
      // set today's date
      this.parent.currentDate = date()
      class::NewMonth()
   return

   function NEXTYEARPUSHBUTTON_onClick
      // code to add year to date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addYears( this.parent.currentDate, 1 )
      class::NewMonth()      
   return

   function YEARPLUS10_onClick
      // code to add 10 years to date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addYears( this.parent.currentDate, 10 )
      class::NewMonth()      
   return

   function YEARMINUS10_onClick
      // code to subtract 10 years from date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addYears( this.parent.currentDate, -10 )
      class::NewMonth()      
   return

   function NEXTMONTHPUSHBUTTON_onClick
      // code to add month to date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addMonths( this.parent.currentDate, 1 )
      class::NewMonth()
   return

   function MONTHPLUS3_onClick
      // code to add 3 months to date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addMonths( this.parent.currentDate, 3 )
      class::NewMonth()
   return

   function PREVMONTHPUSHBUTTON_onClick
      // code to subtract month from date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addMonths( this.parent.currentDate, -1 )
      class::NewMonth()     
   return

   function MONTHMINUS3_onClick
      // code to subtract 3 months from date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addMonths( this.parent.currentDate, -3 )
      class::NewMonth()     
   return

   function PREVYEARPUSHBUTTON_onClick
      // code to subtract year from date,
      // and call newMonth routine
      this.parent.currentDate = ;
           this.parent.dateEx.addYears( this.parent.currentDate, -1 )
      class::NewMonth()
   return

   procedure LoadHolidays
      /*
          The reason we are doing things this way is because
          it seems to be the only way to override the LoadHolidays
          method of the calendar class properly. We want the
          end-user to only have to deal with holiday issues
          in the main kmScheduler.wfm form. We are therefore
          going to go get the Holidays as defined in that form,
          so that the user doesn't have to change anything here.
      */
      local oRef

      // Make sure we have the correct object reference:
      if this.className == "PUSHBUTTON"
         oRef = this.parent
      else
         oRef = this // calendar class
      endif

      // Open the form file:
      set procedure to kmScheduler.wfm additive
      // Create an instance of that form object:
      oSched = new kmSchedulerForm()
      // Load some code/objects:
      oSched.canOpen()
      // Load the holidays:
      oSched.LoadHolidays( oRef.CurrentDate )
      // get the Holiday array:
      oRef.Holidays = oSched.Holidays
      // the form_onClose code in kmScheduler.wfm
      // clears out some objects so we don't have copies
      // hanging out in memory:
      oSched.form_onClose()
      // release the form just to be sure:
      oSched.release()

   Function IsHoliday
      parameter dCheckIt
      // this routine reads the array "this.Holidays"
      // and checks the date to see if the date in
      // dCheck parameter is in the list, and returns
      // a logical value ... 
      if "CALENDAR" $ upper( this.className )
         oRef = this
      else
         oRef = this.parent
      endif
      lHoliday = false
      oRef.cSpeedTip = ""
      for i = 1 to oRef.Holidays.size/2
          if oRef.Holidays[ i, 1 ] == dCheckIt
             lHoliday := true
             oRef.cSpeedTip = oRef.Holidays[ i, 2 ]
          endif
      next
   return ( lHoliday )

endclass
// end of class KenCalendarContainer

/* 
   ----------------------------------------------------------
   Pushbutton class definition:
   ----------------------------------------------------------
*/
class DayPushButton3( oParent ) of PushButton( oParent ) custom
   with (this)
      fontSize := 12
      height   := 1
      width    := 3.2
      metric   := 0  // Chars
      text     := "30"
      pageno   := 0
      visible  := false
      speedBar := true // so they don't keep focus
   endwith

   function onClick
      /*
         The tricky part of this little routine is
         resetting colors -- in order to do that,
         we need to store a reference to the 'current'
         button, so we can reset that, before we change
         the now current button's color ...
      */
      dCurrentDate = new date(year(this.parent.currentDate), ;
                     month(this.parent.currentDate)-1, ;
                     VAL(this.parent.currentButton.text))
      dCurrentDate = ctod( left( dCurrentDate.toLocaleString(), ;
                     iif( set("CENTURY") == "ON", 10, 8 ) ) )
      if type( "this.parent.currentbutton" ) # "U"
         // if it's in Column 1, it's a "sunday" ...
         if "COL1" $ this.parent.currentButton.Name 
            this.parent.currentButton.colorNormal := ;
                                  this.parent.SUNDAYCOLOR
         // set holiday color if needed:
         elseif this.parent.isHoliday( dCurrentDate )
               this.parent.currentButton.colorNormal := ;
                                  this.parent.HOLIDAYCOLOR
         // otherwise, we have a "normal" day ...
         else
            this.parent.currentButton.colorNormal := ;
                                    this.parent.NORMALCOLOR
         endif
      endif

      // Assign currentButton property of container
      this.parent.currentButton = this
      // Assign color ...
      this.colorNormal := this.parent.SELECTEDCOLOR

      // set current date to this one:
      this.parent.currentDate := ;
           new date( year(this.parent.currentDate), ;
                     month(this.parent.currentDate)-1, ;
                     VAL(this.text) )
      this.parent.currentDate := ;
                     ctod( left( this.parent.currentDate.toLocaleString(), ;
                     iif( set("CENTURY") == "ON", 10, 8 ) ) )

endclass

// custom class the "title" text so we have something nearly
// identical to the pushbuttons used for the days for the titles
class TitleText3( oParent ) of Text( oParent ) custom
   with (this)
      fontSize        := 12
      height          := 1
      width           := 3.2
      metric          := 0  // Chars
      text            := "S"
      pageno          := 0
      border          := true
      borderStyle     := 1 // Raised
      alignHorizontal := 1 // center
      alignVertical   := 1 // middle
   endwith

endclass

/*
   ================================================================
   End of file: KenCal3.CC
   ================================================================
*/



