/* -----------------------------------------------------------
   PageXofY.CC
   Author: Mervyn Bick
   Date..: 2016/02/23

   This is a custom text control that will
   generate Page x of y, where 'x' is the current page
   and 'y' is the last page number of the report.
	
	The control is based on the original PageXofY.cc by Ken Mayer 
   but uses a different method of finding the last page.  
   Instead of rendering the report to a print file the report is
   rendered in a hidden reportviewer control.  This speeds up
   the process significanty.  	

   NOTE: If being used in any kind of multi-user situation,
   your report should have all tables in the queries or 
   datamodules locked, as it is possible someone might
   be adding data to the table(s) at the same time you
   are attempting to generate the report (and throwing
   off the page count ...).

   WARNING: This control will (currently) only work as long
   as you do NOT change the name of the pageTemplate, AND
   as long as you do not change the default name of the
   instance of this control placed onto a report -- in other
   words, this must exist: 
	
      reportName.pageTemplate1.PageXofY1

   WARNING 2: If you use this control on a report that
   is displayed in a reportViewer window, scrolling back
   a page will lose focus somehow to some of the methods
   available for the report. The solution is to add the 
   following after the CLASS statement of your report:
   
      set procedure to program(1)
      
      
  Revision. September 2018
   
     Initially the custom control requred that the report's 
     metric be set to 0 (Characters).  Changed code so that 
     the custom control can now be used on a report irrespective 
     of the report's metric.     
     
     Be aware that there is a bug in later versions of the report
     designer which can cause the control to be placed in a position
     where it can't be seen on the report.  If this occurs, select
     the PageXofY object (which will be under the PAGETEMPLATE1 entry)
     in the Inspector and set the top and left properties to 0. 
     This will move the control to the top left corner of the white
     pagetemplate from where it can be dragged to the required position.
     The bug causes the default width of the control to be ignored so
     it will also be necessary to set the control's width such that it 
     can display the full text for the last page.     
      
   ----------------------------------------------------------- */

class PageXofY( parent ) of Text( parent ) custom
   with ( this )
      text        := class::GetPageNum
      OnOpen      := class::GetLastPage
   endwith

   PROCEDURE GetPageNum
      local nPage
      nPage = 0
      if TYPE("this.form.reportPage") == "N" 
         nPage := this.form.reportPage
      endif
      if type( 'this.LastPage' ) = "U" or this.LastPage = 0
         this.LastPage = 9999
      endif
   return ( "Page "+nPage+" of "+this.LastPage )

   PROCEDURE GetLastPage
	   // check to see if last page has been found previously
		if type( "this.dontLoopOnMe" ) # "U"
			return
		endif
      
		// We need a form with a reportviewer
		this.oForm = new getlastform()
		this.oform.parent = this	

      // have to create an instance of the report:
      cReportName = this.form.className
      rRep = new &cReportName.() // new instance of report
		
      this.oform.reportviewer1.ref = rRep
      /*
         -------------------------------------------------------
         find instance of this control, set dontLoopOnMe ...
         assume default (not all controls on report have 'before'
         property, which means we can't loop through controls)
         Have to assume "pageTemplate1" as well as "PageXofY1"
         -------------------------------------------------------
     */
	  
		if type( 'rRep.pageTemplate1.PageXofY1' ) == "O"
			/* 
				Problem: every time you render the report,
				this method will execute, right? If we 
				render it NOW, this method will execute, which
				will attempt to render the report, which will
				cause this method to execute, which will 
				render the report ... nasty. SO, we have to
				have a way to NOT even start that loop:
			*/
			rRep.pageTemplate1.PageXofY1.dontLoopOnMe = true
		endif	  
	   
	
      if type('this.parent.lastpage') # 'N'
		   // Make some changes:
			rRep.startPage      := 1           // we want all 
			rRep.endPage        := -1          //     pages
			rRep.LastPage      = 0            // set counter
		 
			this.oform.open()
		
		   this.lastpage = this.parent.lastpage
				
		endif		
				
				
   return
		
EndClass

class getLastForm of FORM
   with (this)
      onOpen = class::GETLASTPG
      visible = true
      height = 4.4545
      left = 87.7143
      top = 12.2273
      width = 25.0
      text = "Processing"
   endwith

   this.REPORTVIEWER1 = new REPORTVIEWER(this)
   with (this.REPORTVIEWER1)
      height = 1.0
      left = 1.0
      top = 0.5
      width = 1.0
   endwith

   this.RECTANGLE1 = new RECTANGLE(this)
   with (this.RECTANGLE1)
      left = 0.0
      top = 0.0
      width = 25.0
      height = 4.0
      text = " "
   endwith

   this.TEXTLABEL1 = new TEXTLABEL(this)
   with (this.TEXTLABEL1)
      height = 1.0
      left = 6.0
      top = 1.5
      width = 14.0
      text = "Counting pages"
   endwith

   this.TEXTLABEL2 = new TEXTLABEL(this)
   with (this.TEXTLABEL2)
      height = 1.0
      left = 6.0
      top = 2.5
      width = 14.0
      text = 0
   endwith

   function getLastPg
	   local nPage
	   _app.allowYieldOnMsg = true
   	_app.executeMessages()
      lLast = false
      do while lLast = false
         nPage = form.reportviewer1.ref.reportPage + 1
		   form.reportviewer1.ref.startPage = nPage
		   form.reportviewer1.ref.endPage = nPage
		   this.textlabel2.text = nPage
		   form.reportviewer1.ref.render()
         lLast = form.reportviewer1.ref.isLastPage() 
		   _app.executeMessages()
		   if lLast = true
   		   this.parent.lastpage = form.reportviewer1.ref.startpage -1
			   this.parent.parent.lastpage = this.parent.lastpage
		   endif
      enddo
		form.close()
   return

endclass
