// Metric: 0 - Chars or 6 - Pixels only
/*
   ROWSETSPIN.CC

   Programmer..: Wesley E. Rue
   Date........: October 31, 2000
   Notes.......: Used to move to previous or next rowset of a query
                 Features rollover top2bottom and vis versa
   Written for.: dBASE 7.5, dB2K
   Rev. History: October 31, 2000 - Original version (WER)
   Calls.......: None
   Called by...: None
   Usage.......: Place on form, uses form.rowset, no
                 properties need be set (except location,
                 height, width ...)
   Parameters..: None
   Returns.....: None

   Details.....:

   This custom control (CC) is based on the SPINBOX component.
   This CC was developed to work specifically with a rowset and
   grid on the form .  This CC can be dropped on a form first without
   a rowset because of the 'try-catch-endtry' code.

   Basically RowsetSpin.CC is like a pair of vertical VCR Buttons.

   Example:

   In the Command Window:

      set procedure to rowsetspin.cc additive
    
   The CC will be added to the Custom tab of the control palette
   and appear as a 'spinbox' icon.

   Create a form, place an instance of this CC on the form by
   dragging the RowsetSpin 'spinbox' icon from the Custom tab of
   the control palette and dropping it on your form like you 
   would any other component on a form.  Add a 'rowset' and a 
   grid. Place the CC to the right of the grid so that the 
   number portion of the spinbox is covered by the grid and 
   size the CC to heighth.  Set the grid's datalink to the 
   rowset and save the 'test' form.  Recommend setting the 
   grid to only one or two columns for test.

*/
class RowsetSpin(f) of SPINBOX(f) custom
   with (this)
      onChange = class::getPrevNextRowset
      height = 6
      left = 18.5714
      top = 4.8636
      width = 6
      metric = 0 // char
      rangeMax = 1
      rangeMin = -1
      value = 0
   endwith

   function onDesignOpen( lFromPalette )
      if lFromPalette and form.metric == 6 // Pixels
         this.height := 132
         this.width  :=  42
      endif
   return

   function getPrevNextRowset
      try                              // Tests for attached rowset
         if this.value = -1            // RowsetSpin down-arrow click
            if form.rowset.atlast()    // are we at last record
               form.rowset.first()     // Yes, goto first record
            else
               form.rowset.next(1)     // No , goto next record
            endif
         endif
      catch (exception e)
         //-do nothing -- form does not have rowset
      endtry

     try
        if this.value = 1             // RowsetSpin up-arrow click
           if form.rowset.atfirst()   // are we at first record
              form.rowset.last()      // Yes, goto last record
           else
              form.rowset.next(-1)    // No, goto previous record
           endif
        endif
     catch (exception e)
        //-do nothing -- form does not have rowset
     endtry

     this.Value = 0                   // reset RowsetSpin.value = 0
  return

endclass
