//------------------------------------------------------------------------
//
//  structureEx.cc -- class Structure
//  $Revision:   1.1, 10/1999
//
//	 This class is an extension of the structure.prg class
//  delivered as part of VdB 7.01 samples.
//
// NOTE: The path structure is updated as of May, 2017, 
// for StructAPI.prg to make it possible for dBASE to find
// this file in both dBASE Plus 10 and 11 ... Ken Mayer
//
//  It is intented to work with structures passed to 
//  external functions such as the Windows API ones.
//
//	 The following new features have been added or changed to the original ones:
//
//		- TYPE_BYTE,  TYPE_CHAR, TYPE_USHORT, TYPE_INT, TYPE_UINT, TYPE_BOOL, 
//		  TYPE_ULONG, TYPE_FLOAT, TYPE_DOUBLE, TYPE_POINTER, struct member, 
//		  added.
//		- Bit fields structure declarations, like in C++ structures. This allows
//      to handle easily API structures that contain bit fields.
//		- Add a .size property that contains the length of the structure, updated 
//		  when the structure members are added.
//		- Suppress the findIndex function as the member array is now a VdB 
//		  AssocArray. Enhances access to members speed.
//		- the .value property is a string, whose size has been adjusted to best fit 
//		  the actual byte size.
//		- Add a dumpMembers method that displays the content of each member of 
//		  the structure. 
//    - Add an "Ansi" parameter for string and pointers members so that Unicode
//      conversions can be controlled.
//
//  ----------------------------------------------------------------------
//
//  The following example shows how to create a six member
//  structure and use the setMember() and getMember() methods
//  to set and get the value of a structure member.
/*

	- Numeric value
   se = new structExample()
   se.setMember("memberOne", 128)
   ? se.getMember("memberOne") 

   - Bit fields manipulation
   se.setMember( "memberThreeBit56", 0x2 )
   ? se.getMember("memberThreeBit56")

   - String, Unicode in VdB
   se.setMember( "memberTwo", 'abcde' )
   ? se.getMember( "memberTwo" )

   - Pointer, to a byte string in VdB
   se.setMember( "memberTwo", '12345' )
   ? se.getMember( "memberTwo", 'Ansi' )
    
  
//   class structExample of StructureEx from StructureEx.cc
      super::addMember( TYPE_WORD,   "memberOne" )
      super::addMember( TYPE_STRING, "memberTwo", 5 )				  // 5 bytes string
      super::addMember( TYPE_BYTE,   "memberThreeBit1", 1 )      // 1 bit field
      super::addMember( TYPE_BYTE,   "memberThreeBit2-4", 3 )    // 3 bits field
      super::addMember( TYPE_BYTE,   "memberThreeBit56", 2 )     // 2 bits field
      super::addMember( TYPE_BYTE,   "memberThreeBit78", 2 )     // 2 bits field
      super::addMember( TYPE_POINTER,"memberFour", 10 )			  // pointer to 10 bytes
//   endclass

*/ 
//
//  Portions of code Copyright (c) 1997-1998, Borland International, Inc. 
//  All rights reserved.  
//
//  Portions of code Copyright (c) 1999, Nicolas Martin. -  nicolas.martin@freesurf.fr
//
//- Properties:
//  -----------
//
//--  value
//			VdB string, contains the data of the structure.
//			Note: as value is a Unicode string, value.length is different 
//					from the .size property of the object.
//
//--  member
//			VdB AssocArray of objects, each element is an object describing the structure
//			members, based on the StructMember class definition.
//
//--  size
//       VdB numeric, the size of the structure, in bytes.
// 
//- methods:
//  --------
//
//--  addMember( nType, cName, nSize )
//			Use this method to add new members to the structure definition.
//			nType: VdB numeric, can be any type defined in the StructAPI.h file :
//			TYPE_BYTE,  TYPE_CHAR, TYPE_USHORT, TYPE_INT, TYPE_UINT, TYPE_BOOL, 
//		  	TYPE_ULONG, TYPE_FLOAT, TYPE_DOUBLE, TYPE_POINTER
//			cName: VdB String, the member's name as set up by addMember. 
//			nSize: Optional parameter. Only specify this parameter for :
//						- a TYPE_STRING member: size of the string, in bytes
//                - a TYPE_POINTER member: max. # of bytes pointed to
//						- a bit field member: size of the field, in bits
//			return: the member count
//			Note: For each bit field description, the sum of the sizes of each field
//					must be less than 32 bits (i.e. DWORD).
//
//--  setMember( cName, nValue, [cStrType] )
//			This method sets the content of a structure's member value.
//			cName: VdB string, the member's name as set up by addMember.
//			nValue: VdB numeric or string (for TYPE_STRING & TYPE_POINTER), value to set.
//					  Note that for bit fields, only the low order bits corresponding
//					  to the member's size are taken into account. 
//			cStrType : VdB String. Optional parameter.
//						  Used for String members only.
//						  If unspecified, the string is considered Unicode.
//						  If equel to "A" or "Ansi", it's an ANSI Byte string,
//						  so no Byte to Unicode conversion is performed.
//			return: nValue
//
//--  getMember( cName, [cStrType] )
//			This method reads the content of a particular member of the structure.
//			cName: VdB string, the member's name as set up by addMember.
//			cStrType : VdB String. Optional parameter.
//						  Used for String and Pointers members only.
//						  If unspecified, the string returned is convert to Unicode.
//						  If equel to "A" or "Ansi", returns an ANSI Byte string,
//						  without Byte to Unicode conversion.
//			return: the value to read
//			
//--  length( )
//			This method returns the structure's exact size, in bytes
//
//--  dumpMembers
//			This method will get and dump the content of all the members
//			to the result pane of the command window. Usefull for debug 
//			sessions.
//			Members may appear in a different order than when the structure is created
//		  	due to AssocArray.
//			return: members count
//
//------------------------------------------------------------------------  

#include "structAPI.h"

class StructureEx
	protect f1Field, lastKey
   /*
      sFolder = '"' + _DBWINHOME + "Samples\StructAPI.prg" + '"'
      do &sFolder.
   */
   if __version__ => 11
      // dBASE Plus 11, there's a source alias for Samples:
      do :Samples:StructAPI.prg
   else
      // dBASE Plus 10, no alias, so ...:
      sFolder = _app.currentUserPath
      sFolder = left( sFolder, len( sFolder ) - 3 )
      sFolder += "Samples"
      sFolder += "\StructAPI.prg"
      sFolder = '"' + sFolder + '"'
      do &sFolder.
   endif
      
   
   // Prototype GetStructString into GetStructByteString to create a new 
   // function that allows to get Byte strings, without Unicode conversion.
   extern CINT GetStructByteString(CPTR, CINT, CINT, CPTR, CINT) ;
      strucmem.dll from "GetStructString"
   // Prototype SetStructString into SetStructByteString to create a new 
   // function that allows to set Byte strings, without Unicode conversion.
   extern CINT SetStructByteString(CPTR, CINT, CINT, CPTR, CINT) ;
      strucmem.dll from "SetStructString"

	this.member = new AssocArray()			// member array changed to AssocArray
	this.value = ""
	// Add : Size and f1Field properties to deal with bit fields members
	this.Size = 0
   this.lastKey = Null			// The last key added

	// This functions sets up a new member in the structure. Major modifications
   // since original version.
	function addMember( nType, sName, nSize )
		local  i, oCur, oPrev
		nSize = IIF(PCOUNT()==3,nSize,0)
		this.member[ sName ] = ( new StructMemberEx( nType, nSize ))
		oCur = this.member[ sName ]
		i =  this.member.count( )
		if ( i > 1 )
			oPrev = this.member[ this.lastKey ]
         do case
         	case ( nType == TYPE_POINTER ) or ( nType == TYPE_STRING )
               oCur.offset := oPrev.offset + oPrev.TypeSize
               this.f1Field := true

            otherwise		// Numeric type
					if ( nSize == 0 )			// Not a bit field
	               oCur.offset := oPrev.offset + oPrev.TypeSize
               elseif this.f1Field		// First bit field
                  oCur.offset := oPrev.offset + oPrev.TypeSize
                  this.f1Field := false
               else							 // Next bit field
                  oCur.offset := oPrev.offset
                  oCur.bitShift := oPrev.bitShift + oPrev.size
               endif
               if ( oCur.bitShift + oCur.size ) >= oCur.TypeSize	// Last bit field ?
                  this.f1Field := true
               endif
         endcase
		else
      	this.f1Field = ( nSize == 0 ) or ( nType == TYPE_POINTER ) or ( nType == TYPE_STRING )
		endif
		this.lastKey = sName
		this.Size = oCur.offset + oCur.TypeSize
      // Sets the exact size (modulo 1 word) for the structure contents
		this.value := REPLICATE( chr( 0 ), ceiling( this.Size / 2 ))

// For debug purposes
// ? this.Size
// ?? oCur.offset, oCur.size, oCur.bitShift, sName

		return ( i )

   // retreives the value of a structure member.
	function getMember( sName, cStrType )
		local xValue, nBytes, nType, nOffset, nSize, n8BitShift, oCur, cStrType1
		nBytes = 0
		oCur = this.member[ sName ]
		nType   = oCur.type
		nOffset = oCur.offset
		nSize   = oCur.size
		n8bitShift = oCur.bitShift * 8
		do case
			case ( nType == TYPE_STRING )
		      cStrType1 = iif( argcount( ) == 2, cStrType, 'U' )
            if cStrType1.left( 1 ).toUpperCase( ) == "A"		// ANSI, so no conversion
               xValue = REPLICATE( chr( 0 ), ceiling( nSize / 2 ))
               nBytes = GetStructByteString( this.value, nOffset, nSize, xValue, nSize)
				else			// Unicode conversion
               xValue = REPLICATE(" ", nSize)
					nSize  = GetStructStringLen( this.value, nOffset )
					nBytes = GetStructString( this.value, nOffset, nSize, xValue, nSize)
            endif

			case ( nType == TYPE_POINTER )
		      cStrType1 = iif( argcount( ) == 2, cStrType, 'U' )
            if cStrType1.left( 1 ).toUpperCase( ) == "A"		// ANSI, so no conversion
               xValue = REPLICATE( chr( 0 ), ceiling( nSize / 2 ))
               nBytes = GetStructVoidPointer( this.value, nOffset, nSize, xValue, nSize)
				else			// Unicode conversion requested
               xValue = REPLICATE(" ", nSize)
					nBytes = GetStructCharPointer( this.value, nOffset, nSize, xValue, nSize)
            endif

			otherwise       // Numeric type
				xValue = GetStructNumber( this.value, nOffset, nType )
				
				if ( nSize # oCur.typeSize )           // this is a bit field
					if ( n8bitShift > 0 )               // Not first field, so shift
						xValue := bitzrshift( xValue, n8bitShift )
					endif
					xValue := bitand( xValue, bitlshift( 1, nSize * 8 ) - 1 )
				endif
			endcase                     
	return ( xValue )


	// sets the value of a particular member.
	function setMember( sName, xValue, cStrType )
		local nBytes, nType, nOffset, nLength, nBitShift8, yValue, oCur, i, cStrType1
		nBytes = 0
		oCur = this.member[ sName ]
		nType   = oCur.type
		nOffset = oCur.offset
		nSize   = oCur.size
		nLength = this.Size
		nBitShift8 = oCur.bitShift * 8
		do case
			case ( nType == TYPE_STRING )
				if ( PCOUNT() == 1 )
               // xValue changed to yValue so that the input param remains unchanged
					yValue = REPLICATE(" ",nSize-1) + CHR(0)
				else
					yValue = SUBSTR(xValue,1,nSize-1) + CHR(0)
				endif
		      cStrType1 = iif( argcount( ) == 3, cStrType, 'U' )
            if ( cStrType1.left( 1 ).toUpperCase( ) == "A" )
					nBytes := SetStructByteString( this.value, nOffset, nLength, yValue, nSize )
            else
					nBytes := SetStructString( this.value, nOffset, nLength, yValue, nSize )
            endif

			case ( nType == TYPE_POINTER )
         	// Write the pointer to xValue in the structure
				nBytes := SetStructVoidPointer( this.value, nOffset, nLength, xValue )

			otherwise			// Numeric type
				if ( nSize # oCur.typeSize )          // bit field case
					// Get the current value
					yValue = GetStructNumber( this.value, nOffset, nType )	
               // Set the corresponding field to zero
					yValue := bitand( yValue, bitnot( bitlshift( ; 
               						bitlshift( 1, nSize * 8 ) - 1, nBitShift8 )))
               // Set the corresponding field to its new value
					yValue := bitor( yValue, bitlshift( xValue, nBitShift8 ))
				else
            	yValue = xValue
				endif
				nBytes := SetStructNumber( this.value, nOffset, nType, nLength, yValue )  
         endcase                     
   return ( xValue )

   function Length
   	return this.Size

	// Added: method to dump the content of a structure.
	// NOTE : Members may appear in any order, as for AssocArrays.
	function dumpMembers
		local i, cKey
		private xValue
		cKey = this.member.firstKey
		for i = 1 to this.member.count( )
			xValue =  this.getMember( cKey )
			if type( 'xValue' ) == 'N'
				? cKey, ': 0x' + itoh( xValue ), '= ' + xValue
			else
				? cKey, ':', xValue
			endif
			cKey := this.member.nextKey( cKey )
		endfor
		return ( this.member.count( ) )

endclass

// Modified class for structure members description. The Name is now used as
// index for the AssocArray and is thus suppressed from StructMember.
class StructMemberEx( nType, nSize )
	this.type   = nType
	this.size   = 0
	this.offset = 0
	this.bitShift = 0
   this.TypeSize = 0
	do case
		case ( nType == TYPE_BYTE ) 
			  this.size := SIZEOF_BYTE
		case ( nType == TYPE_CHAR )
			  this.size := SIZEOF_CHAR
		case ( nType == TYPE_SHORT )
			  this.size := SIZEOF_SHORT
		case ( nType == TYPE_USHORT )
			  this.size := SIZEOF_USHORT
		case ( nType == TYPE_WORD )
			  this.size := SIZEOF_WORD
		case ( nType == TYPE_INT )
			  this.size := SIZEOF_INT
		case ( nType == TYPE_UINT )
			  this.size := SIZEOF_UINT
		case ( nType == TYPE_DWORD )
			  this.size := SIZEOF_DWORD
		case ( nType == TYPE_BOOL )
			  this.size := SIZEOF_BOOL
		case ( nType == TYPE_LONG )
			  this.size := SIZEOF_LONG
		case ( nType == TYPE_ULONG )
			  this.size := SIZEOF_ULONG
		case ( nType == TYPE_FLOAT )
			  this.size := SIZEOF_FLOAT
		case ( nType == TYPE_DOUBLE )
			  this.size := SIZEOF_DOUBLE
		case ( nType == TYPE_POINTER )
			  this.size = SIZEOF_POINTER
		case ( nType == TYPE_STRING )
			  this.size := nSize
	endcase

	// Added for bit fields
	this.TypeSize = this.size
   // condition for a bit field
	if ( nSize > 0 ) and ( nType <> TYPE_STRING ) and ( nType <> TYPE_POINTER )	
		this.size := nSize / 8			// Size, in bytes, of the bit field
   endif
   // Added for TYPE_POINTER
	if ( nType == TYPE_POINTER )
   	this.size := nSize
   endif
endclass
