/*
XtraBtns.CC
Extra buttons
By Peter Rorlick and Saveli Kossenko
Copyright MBS
*/
***********************************************************************
CLASS POPBUTTON(FormObj) OF FlashBUTTON(FormObj) Custom

* As of March 2 1998, this inherits FlashButton's logic, which we prefer
* because it doesn't set focus to the button in the OnMouseMove.

/*
* Simple version: inherits from zPushbutton

* Set focus to the button automatically if mouse is moved over it.
* The button turns yellow when in focus.
* If the form is a subclass of zForm, moving the mouse over a PopButton
* does nothing if the form does not have focus.
* lIsInFocus is handled by the zForm class in zControl.cc

   with (this)
     OnMouseMove = {; If type('form.lIsInFocus')#'L' .or. form.lIsInFocus ; this.SetFocus() ; Endif }
     OnGotFocus =  {;this.ColorNormal = 'N/RG+'}
     OnLostFocus = {;this.ColorNormal = 'BtnText/BtnFace'}
     ColorNormal = "BtnText/BtnFace"
     Metric = 0
     Group = true
     Value = false
     Height = 1.3
     Width = 12
   endwith
*/
   this.Text = "PopButton"

EndClass
******************************************************************
*
* FLASHBTN.CC
*
* Reverses the color of the pushbutton when the mouse
* cursor passes over it.
*
* Author: Gary White  garyw@qni.com
*
* Last modified: Oct 06, 1997
* Then by Peter Rorlick on March 2, 1998
*
* Usage: In the form designer, click File, Set up custom controls
* and add the FlashButton.CC.  Then just select the FlashButton
* from the Custom tab of your control palette and install it just
* like you would any other button.
*
* This custom class uses the pushbutton's OnOpen event to install
* the routines.  Do NOT overwrite the FlashButton's OnOpen event.
* The OnMouseMove event triggers the color change and is protected.
*
* Note that any controls created after the form's OnOpen will not
* have the routine installed to return the FlashButton to it's
* normal colors.  This should rarely be a problem unless the newly
* created control is touching the FlashButton, in which case the
* button will not return to normal until the mouse pointer moves
* off of the new control.  Unlikely, but it could happen.
*
* Thanks to Romain Strieff and Peter Rorlick for finding my errors
* and suggesting fixes and improvements.
*
* Enjoy!
* Gary
*
/*
Changes made by Peter Rorlick on March 2nd:

We now test whether a control is a Flashbutton or one of its
subclasses:
   if Type('oControl.HotColor')=='U'
This allows us to subclass it without it breaking.

I've modified the Do...Until loop so that it recurses through
the objects that belong to containers and notebooks.
This solves the small glitch that occurred when a
flashbutton resided on a container or notebook and there was
another control touching it: when the mouse moved from the
flashbutton to the other control, the button did not un-flash.

The March 2 version now sets a specific hot color (Black/Yellow) and
changes the borderstyle, instead of simply reversing the color.
I find that it looks much nicer. But if you have small buttons with
bitmaps, you might prefer to comment out the lines that change the
borderstyle (or set borderStyle to 4 rather than 8).

We now use an Init method instead of OnOpen, so that we don't have
to worry about programmers over-writing the OnOpen.  To take advantage of
the Init method, the form must be based on zForm or one of its subclasses.
If not based on zForm, it uses the OnOpen method.

The button also lights up now when the user tabs to it.
*/
*******

CLASS FLASHBUTTON( FormObj ) OF PUSHBUTTON( FormObj ) Custom
   protect OnOpen, OnMouseMove
   this.Height       = 1.3
   this.Width        = 12
   this.Text         = "FlashBtn"
   this.FontBold     = .f.
   this.OnMouseMove  = class::FlashButton_OnMouseMove
   this.OnGotFocus   = class::FlashButton_OnMouseMove
   this.SingleBorder = false  // Override this to True to prevent wrapping of text when button is lit up.
   If Type('form.Init') # 'U'
     * If the form is subclassed from zForm, it will automatically
     * run any Init methods of the controls on the form, before the form
     * opens.  In this case, We use the Init method instead of the OnOpen,
     * to "free up" the button's OnOpen for other purposes that may be
     * desired by the programmer.
   Else
     * Form is not subclassed from zForm. In this case, remember to call
     * Flashbutton::Init() when you over-write the button's OnOpen.
     this.OnOpen = class::Init
   Endif
   this.parent.OnMouseMove = {;class::FlashButton_BlinkOff()}  // For Containers and Notebooks that contain flashbuttons.
   form.OnMouseMove = {;class::FlashButton_BlinkOff()}   // Needed when the only flashbuttons on the form are on Containers or Notebooks.


   Procedure FlashButton_OnMouseMove( flags, row, col )
      * Make the button stand out visually:
      *
      * Problem: When borderStyle becomes 8, it sometimes causes the button's text to wrap.
      *   We see this more often in French - the form is designed in English
      *   but the translated French text can come closer to the border.
      * Solution: If _app.language exists and is not "English", then let the 
      *   "hot" BorderStyle be 4 instead of 8.
      *   Alternatively, in the button's OnOpen, you can do this: {;this.SingleBorder=.t.}
      *
      if type("form.HotButton") # "O"
        this.ColorNormal = this.HotColor
        if this.SingleBorder or ( type('_app.language')=='C' and  _app.language#'English' )
          this.borderStyle = 4   //  Single
        else
          this.borderStyle = 8   //  Modal
        endif
        form.HotButton = this
      elseif this.Name # form.HotButton.Name
        form.HotButton.BorderStyle = 0 // back to default
        form.HotButton.ColorNormal=form.HotButton.NormalColor
        if type('_app.language')=='C' and  _app.language#'English'
          this.borderStyle = 4   //  Single
        else
          this.borderStyle = 8   //  Modal
        endif
        this.ColorNormal = this.HotColor
        form.HotButton = this
      endif


   Procedure FlashButton_BlinkOff
      * Return the button's appearance to normal (un-flash it):
      if type("form.HotButton") == "O" and form.HotButton#this
         form.HotButton.ColorNormal=form.HotButton.NormalColor
         form.HotButton.BorderStyle = 0 // back to default
         form.HotButton=""
      endif

***
* The Init (OnOpen) routine installs the BlinkOff routine in all other
* controls on the form.  It also determines the NormalColor from
* the ColorNormal assigned by the programmer.
***

   Procedure Init  // was: FlashButton_OnOpen
      if type('this.hotColor') == 'C'
        * We assume the form is being re-opened.
        * No need to do anything here.
        return
      endif
      private oControl
      this.NormalColor=this.ColorNormal
*      this.HotColor=right(this.ColorNormal, len(this.ColorNormal) - ;
         rat("/", this.ColorNormal)) +;
         "/" + left(this.ColorNormal, at("/", this.ColorNormal) -1)
      this.HotColor = 'Black/Yellow'
      if type("form.FlashBtnInstalled") == "U"
         form.FlashBtnInstalled=.t.
         CLASS::Install_BlinkOff(Form)
      endif


   Proc Install_BlinkOff(InitialObject)
   private o
   o = InitialObject.First
   do
     if Type('o.First')=='O' and type('o.UseTablePopUp')=='U'  // if o is a container or notebook (but not a form, because forms have a UseTablePopUp property) that contains other objects...
       CLASS::Install_BlinkOff(o)   // recurse through the objects of the container or notebook
     EndIf
     if type( "o.onMouseMove" ) # "U"
       *if upper(o.ClassName) # "FLASHBUTTON"   // This line breaks when FlashButton is subclassed.
       if Type('o.HotColor')=='U'   // If this is NOT a flashbutton or one of its subclasses...
         if empty(o.OnMousemove)
           o.OnMouseMove = {;class::FlashButton_BlinkOff()}
         else
           o.sOnMouseMove = o.OnMouseMove
           o.FlashButton_Blinkoff = {;class::FlashButton_Blinkoff(); this.sOnMouseMove()}
           o.OnMouseMove = {;this.FlashButton_Blinkoff()}
         endif
       endif
     endif
     o = o.before
   until o.name == InitialObject.First.name
   return

ENDCLASS
***********************************************************************
CLASS OKPopBUTTON(FormObj) OF POPBUTTON(FormObj) Custom
   with (this)
     text = "O&K"
     SpeedTip = "Click this button to save changes and close the form"
     StatusText = "Save changes and close the form."
     UpBitmap = "RESOURCE #36"
     Height = 1.3
     Width = 12
   endwith

EndClass
***********************************************************************
CLASS CancelPopBUTTON(FormObj) OF POPBUTTON(FormObj) Custom
   with (this)
     onClick := {; form.close()}
     text="&Cancel"
     SpeedTip="Click this button to discard changes and close the form"
     StatusText="Discard changes and close the form."
     UpBitmap="RESOURCE #28"
   endwith
EndClass
***********************************************************************
class NewPopButton(parentObj) of POPBUTTON(parentObj) custom
   with (this)
      text = "&New"
      upBitmap = "RESOURCE #2060"
      SpeedTip = "Add a new record"
   endwith
endclass
***********************************************************************
class EditPopButton(parentObj) of POPBUTTON(parentObj) custom
   with (this)
      text = "&Edit"
      upBitmap = "RESOURCE TL_EDIT"
      SpeedTip = "Edit the highlighted record"
   endwith

	Function IsARowHighlighted
     * Note: You will always want to call this at the start of your OnClick method,
     * for example:
     *   if not EditPopButton::IsARowHighlighted()
     *     return
     *   endif
     *
     * Check whether a record is highlighted, and if not then tell the user:
     If empty(Form.Rowset)   // if xbase DML...
       if empty(alias())
         TMsgBox("Table isn't open","Error",16)
         Return .f.
       elseif eof()
         TMsgBox("You must hightlight a record first.","Error",16)
         Return .f.
       EndIf
     Else                   // we have a rowset...
       private r
       r = iif( type('this.rowset')=='O', this.rowset, form.rowset )
       * This.Rowset is an optional custom property, to be used when
       * the rowset being modified is NOT form.rowset.  For an example,
       * see RWGrid.CC.
       if r.endOfSet
         TMsgBox("You must hightlight a record first.","Error",16)
         return .f.
       endif
     EndIf
	return true

endclass
***********************************************************************
class ReplicatePopButton(parentObj) of POPBUTTON(parentObj) custom
   with (this)
      text = "&Replicate"
      upBitmap = "RESOURCE #2040"
      SpeedTip = "Add a new record that is a copy of the highlighted one"
   endwith

   * Note: You will always want to do the following at the start of your 
   * OnClick method:
   *   if not EditPopButton::IsARowHighlighted()
   *     return
   *   endif

endclass
***********************************************************************
CLASS DeletePopBUTTON(FormObj) OF POPBUTTON(FormObj) Custom
   with (this)
     OnClick=CLASS::DeletePopBUTTON_OnClick
     Text="&Delete"
     SpeedTip="Delete the highlighted record"
     StatusText="Delete record"
     UpBitmap="RESOURCE #2240"
   endwith

	proc DeletePopBUTTON_OnClick
     If empty(Form.Rowset)   // if xbase DML...
       if empty(alias())
         TMsgBox("Table isn't open","Error",16)
         Return
       elseif eof()
         TMsgBox("You must hightlight a record first.","Error",16)
         Return
       elseif TMsgBox("Are you sure that you want to delete this record?","Confirmation",36) = 6
         UpdateModStamp(form)
         delete
         skip
         if eof()
           go top
         endif
       EndIf
     Else                   // we have a rowset...
       private r
       r = iif( type('this.rowset')=='O', this.rowset, form.rowset )
       * This.Rowset is an optional custom property, to be used when
       * the rowset being modified is NOT form.rowset.  For an example,
       * see RWGrid.CC.
       if r.endOfSet
         TMsgBox("You must hightlight a record first.","Error",16)
         return
       Else
         if r.parent.className == 'ZQUERY'
           * ZQuery's rowset has built-in logic to handle the are-you-sure
           * prompting of the user, plus post-deletion navigational logic.
           r.Delete()
         else    // else we assume the rowset was derived from a native Query object....
           r.NativeDelete = r.Delete  // Need this for the DeleteAndFlush method.
           r.DeleteAndFlush = {; zQuery::DeleteAndFlush() }
           r.DeleteAndFlush()  // Prompts user to confirm, does navigation.
         endif
       endif
     EndIf
	return
ENDCLASS
***********************************************************************
CLASS ClosePopBUTTON(FormObj) OF POPBUTTON(FormObj) Custom
   with (this)
     OnClick={;Form.Close()}
     text="&Close"
     SpeedTip="Click this button to close the form"
     StatusText="Close the form."
     UpBitmap="RESOURCE #1005"
   endwith
EndClass
***********************************************************************
* SButton is a small square button that has no text and does not retain focus - e.g., for for Calendar.wfm.
CLASS SBUTTON(FormObj) OF PUSHBUTTON(FormObj) Custom
	With (this)
      height = 1.25 // was 1.2
      width = 3.55 // was 3.5
      metric = 0
      tabStop = false
      group = true
      value = false
      text = " "
      SpeedBar = true
   EndWith
EndClass
***********************************************************************
CLASS PrintPopBUTTON(FormObj) OF POPBUTTON(FormObj) Custom
   with (this)
      OnClick = {; this.form.PrintIt()}
      text = "&Print"
      upBitmap = "RESOURCE TS_PRINT"
      SpeedTip="Click this button to print current form"
      StatusText="Print form."
   endwith
EndClass
***********************************************************************
CLASS FIRSTPopBUTTON(FormObj) OF POPBUTTON(FormObj) Custom
   with (this)
      width = 8
      text = "&First"
      upBitmap = "RESOURCE TS_FIRST"
      SpeedTip="Click this button to go to the First record"
      StatusText="First record."
   endwith
EndClass
***********************************************************************
CLASS PrevPopBUTTON(FormObj) OF POPBUTTON(FormObj) Custom
   with (this)
      width = 8
      text = "&Prev."
      upBitmap = "RESOURCE TS_PREV"
      SpeedTip="Click this button to go to the Previous record"
      StatusText="Previous record."
   endwith
EndClass
***********************************************************************
CLASS NextPopBUTTON(FormObj) OF POPBUTTON(FormObj) Custom
   with (this)
      width = 8
      text = "&Next"
      upBitmap = "RESOURCE TS_NEXT"
      SpeedTip="Click this button to go to the Next record"
      StatusText="Next record."
   endwith
EndClass
***********************************************************************
CLASS LASTPopBUTTON(FormObj) OF POPBUTTON(FormObj) Custom
   with (this)
      width = 8
      text = "&Last"
      upBitmap = "RESOURCE TS_LAST"
      SpeedTip="Click this button to go to the Last record"
      StatusText="Last record."
   endwith
EndClass
***********************************************************************
CLASS VCRPOPContainer(FormObj) OF CONTAINER(FormObj) Custom
   With (this)
     Width=35//48
     Height=1.3
     Metric = 0	// Chars
     borderStyle = 3	// None
   EndWith

   this.FIRSTPOPBUTTON1 = new FIRSTPOPBUTTON(this)
   with (this.FIRSTPOPBUTTON1)
      OnClick=CLASS::FIRSTPOPBUTTON1_OnClick
      left = 0
      top = 0
   endwith

   this.PREVPOPBUTTON1 = new PREVPOPBUTTON(this)
   with (this.PREVPOPBUTTON1)
      OnClick=CLASS::PREVPOPBUTTON1_OnClick
      left = 9
      top = 0
   endwith

/*   this.RECNO1= NEW TEXT(this)
   with (this.RECNO1)
      left=18
      top=0
      width=10
      text="Rec: 1/70"
   endWith*/

   this.NEXTPOPBUTTON1 = new NEXTPOPBUTTON(this)
   with (this.NEXTPOPBUTTON1)
      OnClick=CLASS::NEXTPOPBUTTON1_OnClick
      left = 18//30
      top = 0
   endwith

   this.LASTPOPBUTTON1 = new LASTPOPBUTTON(this)
   with (this.LASTPOPBUTTON1)
      OnClick=CLASS::LASTPOPBUTTON1_OnClick
      left = 27
      top = 0
   endwith

   proc FIRSTPOPBUTTON1_OnClick
     If EMPTY(Form.Rowset)

		  If EMPTY(alias())
          MsgBox("Table isn't open","Error",16)
          Return
        EndIf

       If RECNO()>1
     	   GO TOP
       Else
         MsgBox("Already at the first record","Error",16)
       EndIf
     ELSE
       if (Form.Rowset.atFirst())
         MsgBox("Already at the first record","Error",16)
       Else
         Form.Rowset.first()
       EndIf
     EndIf
   retu

   proc PREVPOPBUTTON1_OnClick
     If EMPTY(Form.Rowset)

		  If EMPTY(alias())
          MsgBox("Table isn't open","Error",16)
          Return
        EndIf

       If RECNO()>1
         SKIP -1
       Else
         MsgBox("Pointer is at the first record","Error",16)
       EndIf
     ELSE
       if (Form.Rowset.atFirst())
         MsgBox("Pointer is at the first record","Error",16)
       Else
         Form.Rowset.next(-1)
       EndIf
     EndIf
   retu

   proc NEXTPOPBUTTON1_OnClick
     If EMPTY(Form.Rowset)

		  If EMPTY(alias())
          MsgBox("Table isn't open","Error",16)
          Return
        EndIf

       If RECNO()<RECCOUNT()
         SKIP
       Else
         MsgBox("Pointer is at the last record","Error",16)
       EndIf
     ELSE
       if (Form.Rowset.atLast())
         MsgBox("Pointer is at the last record","Error",16)
       Else
         Form.Rowset.next(1)
       EndIf
     EndIf
   retu

   proc LASTPOPBUTTON1_OnClick
     If EMPTY(Form.Rowset)

		  If EMPTY(alias())
          MsgBox("Table isn't open","Error",16)
          Return
        EndIf

       If RECNO()<RECCOUNT()
         GO BOTTOM
       Else
         MsgBox("Already at the last record","Error",16)
       EndIf
     ELSE
       if (Form.Rowset.atLast())
         MsgBox("Already at the last record","Error",16)
       Else
         Form.Rowset.last()
       EndIf
     EndIf
   retu

EndClass
***********************************************************************

