/*
   ---------------------------------------------------------------------
   Program.....: TestVDBMail.prg
   Author......: Ken Mayer
   Date........: January, 2021
   Notes.......: This program is really meant to be used to test the
                 code for Marc Van den Berghen's vdbmail dll. It should
                 only be run from the desktop, but it breaks down and explains
                 the various options, please read through the code 
                 as well as the explanation in the tutorial. This does not
                 need to be build into a .exe, it is only for testing
                 (and registration of the .dll with Windows).
   Written for.: dBASE 2019
   Rev. History: January, 2021  1.0 - Original version (KJM)
   Calls.......: mv_mail.cc -- Marc's custom class for interacting
                               with the dll file.
   ----------------------------------------------------------------------------
   Program copied from the dBASE Web Tutorial, it is a more detailed
   breakdown of how to use the vdbmail software by Marc than his mailtest
   program is. However you need to make a lot of changes here to use it,
   read the comments!!
   ---------------------------------------------------------------------
*/

// open the mv_mail.cc file -- if the path is different, please update
// this statement as well as the "close procedure" at the end of the program:
// set the path to where "mv_mail.cc" and the other programs are:
set procedure to mv_mail.cc

// create an instance of the mv_mail custom class
oMail= new mv_mail()

// oleautoclient not yet registered or unable to to create ...
if oMail.errorcode = 371 or oMail.errorcode = 370
   // provide the path where the vdbmail dll and tlb files are, and
   // since ultimately we're running our code through Apache,
   // which is running either as administrator or as a service,
   // and defaults to administrator, we are stating to register
   // this as an administrator. If you need this for a desktop
   // application only, you can leave out the second parameter,
   // which will default to false ...
   bSuccess = oMail.register( "vdbmail_path", true )
   // if running this program in the same folder as the
   // .dll/.tlb files:
   // bSuccess = oMail.register()
   // will work fine as the current path will be used. If you
   // run this from a DIFFERENT path, then you need the path.
   // The only reason for using the second parameter is if you
   // are doing (as noted above) a web application, where the
   // code must be run as administrator or as a service ...
else
   bSuccess = true
endif

if not bSuccess 
   ? oMail.errortext // registration failed or problem creating OLE object
   return 
endif

try

   // ***********************************************************
   // set server required information:
   // ***********************************************************
   // host is the internet service provider's host information
   // for sending email programmatically:
   oMail.setHost( "" )
   // port defaults to 25, for non SSL web servers (secure socket layer)
   // and to 587 for secure. However, not all web servers are set the
   // same, you need to check with your ISP
   oMail.setPort( 587 )
   // the username, may be an email address, or it may be something 
   // else. Your ISP should be able to tell you
   oMail.setUser( "" )
   // Set the Password for the username above
   oMail.setPW( "" )
   // logical value for SSL -- defaults to true, if it is, you MUST
   // provide the user and password, otherwise you can leave those out.
   oMail.setSSL( true )
   
   // ***********************************************************
   // set the individual mail data
   // ***********************************************************
   // Who is the email coming from? Must be an email address,
   // note that you can optionally add a display name by using a 
   // backslash followed by the name, such as:
   //     oMail.setFrom( "admin@mycompany.com\Web Administrator" )
   oMail.setFrom( "" )
   // If you expect a reply, you can set this address as the
   // same who it is "from" (see above), or you can set it to
   // a completely different email address. NOTE: the
   // ReplyTo does not understand the display name and 
   // will return an error if you use something as above
   // with the "setFrom()" method's comments.
   oMail.setReplyTo( "" )
   // The Subject for the email. These are important and should not
   // be ignored. 
   oMail.setSubject( "" )
   // The body of the email, if the setHTMLBody() method is
   // set to true, you can use HTML to format the body of the 
   // email message. For something complex, you might want to
   // use a variable to build the string and just put the
   // name of the variable inside the parens: oMail.setBody( cEMailBody )
   oMail.setBody("")
   // determines if you are sending email with HTML formatting.
   // Note that some email servers (not as common but they still
   // exist) do not pass along HTML:
   oMail.setHTMLBody( true )
   // Who are you sending this to? 
   //   You can optionally add a display name with a backslash
   //   followed by the name:
   //     oMail.setFrom( "fflintstone@mycompany.com\Fred Flintstone" )
   //     This should work with the arrays below as well.
   // For more, See comments below:
   oMail.setAddress("")
   // Some notes on this: 
   // You can use multiple email addresses in two ways:
   //    Using an array, and there are two options here: a literal array
   //    or an array object, both work:
   //       // literal array:
   //       oMail.setAddress( { "first_email@somewhere.com", "second_email@somewhereelse.com" } )
   //       // array object:
   //       aEmail = new array()
   //       aEmail.add( "first_email@somewhere.com\First Email" )
   //       aEmail.add( "second_email@somewhereelse.com\Second Email" )
   //       oMail.setAddress( aEMail )
   //    or you can simply call the method multiple times:
   //       oMail.setAddress( "first_email@somewhere.com" )
   //       oMail.setAddress( "second_email@somewhereelse.com" )
   // whew. 

   // ***********************************************************
   // optional settings with descriptions:
   // ***********************************************************
   // Add attachments to the email. No array is allowed for this,
   //   so if you need more than one, call the method more than once:
   // oMail.setAttachment( "mv_mail.cc" )
   
   // Priority defaults to 1 = "Normal", but you can set the priority
   // to 0 for "Low" priority, or 2 for "High" priority. If the email
   // client the email receiving the email uses supports priority
   // settings, this is what triggers that:
   // oMail.setPriority( 1 ) // 1 is default

   // CC: -- repeat this for each address you want to add
   // as a courtesy copy -- the display name works the same
   // as the examples above, using a backslash followed by
   // the name after the email address:
   // oMail.setCC( "valid email" )
   
   // BCC: -- same as above, but the method is named bcc
   //  -- the display name works the same
   // as the examples above, using a backslash followed by
   // the name after the email address:
   // oMail.setBCC( "valid email" )
   // hint, use this for a large list of recipients, so a reply or 
   // reply all does not spam everyone getting the email
   
   // You can request an "email was read" receipt, and if the
   // person reading the message says it is okay, it will send
   // you a note that the message was read, the date, and time.
   // You have to provide an email address to send the receipt to.
   // oMail.setReadReceipt( "valid email" )
   
   // Request an email message was "received" by the user at their
   // inbox, this one the user doesn't see the request, is not
   // asked about, but it doesn't tell you if they read it, just
   // that it arrived ...
   //oMail.setReceiveReceipt( "valid email" )
   
   // Send the email
   oMail.send() 

   if oMail.errorCode > 0
      ? oMail.errorcode , oMail.errorText
   else
      ? "EMail sent, check your inbox ..."
   endif
catch ( exception e )
   ? e.code, e.message
endtry

// cleanup
release object oMail
oMail := null
// path to where the vdbmail files are:
close procedure mv_mail.cc