/*
   ListAllTables.prg

   Uses the BDEAlias.cc in the dUFLP, to list all tables in
   all BDE Aliases defined currently. It returns a three column
   array, with the Alias, the path, and the tablename.

   Usage: set procedure to :dUFLP:ListAllTables.prg additive
          aTest = ListAllTables()
          for i = 1 to aTest.size/3
              ? aTest[i,1], aTest[i, 2], aTest[i, 3]
          next

   Ken Mayer
   April 26, 2002

   Uses: :dUFLP:BDEAlias.cc
*/

function ListAllTables
   set procedure to :dUFLP:BDEAlias.cc additive
   oBDE = new BDEAlias()

   // Get a list of all aliases and paths:
   aAliases = oBDE.ListAliases( true ) // get paths

   // no aliases defined:
   if aAliases.size == 0
      return false
   endif
   // sort
   aAliases.sort()
   aAllTables = new array( 1, 3 ) // three column array
   nRows = 1

   // Loop through aliases array
   for i = 1 to aAliases.size/2
       aTables = oBDE.ListTables( aAliases[ i, 1 ] )
       aTables.sort()
       for j = 1 to aTables.size

           // if we have only one row in aAllTables, check
           // to see what the values are:
           if nRows == 1 and aAllTables[1,1] == false
              aAllTables[1,1] = aAliases[ i, 1 ] // Alias
              aAllTables[1,2] = aAliases[ i, 2 ] // Path
              aAllTables[1,3] = aTables[ j ]     // Table
           else
              nRows++
              aAllTables.resize( nRows, 3, 1 ) // add a row and don't rearrange
              aAllTables[ nRows, 1 ] = aAliases[ i, 1 ]
              aAllTables[ nRows, 2 ] = aAliases[ i, 2 ]
              aAllTables[ nRows, 3 ] = aTables[ j ]
           endif
       next
   next

   // done:
   release object oBDE
   oBDE = null
   release object aTables
   aTables = null
   release object aAliases
   aAliases = null
   close procedure :dUFLP:BDEAlias.cc

// return the array:
return aAllTables

