/*
  ----------------------------------------------------------------
   MISCCODE.PRG

   This program will contain some dBASE Code that doesn't
   really fit into any specific category ...

   November 24, 2004

   Updated: 

   Included here:

      GetDesktopFolder -- returns path to the desktop folder.

   	keyExists() -- used to replace the KeyMatch() function in
                     XDML, based on code by Gary White ...

      roundPrec() -- Precision Rounding -- this function deals with
                     what is commonly called the "rounding error"
                     in dBL ... (it's actually an issue with the
                     way numbers are represented in C++, which is
                     what dBASE is written in, and not really a bug,
                     although it can be quite frustrating ...).

      WhatOS()    -- Returns name of current operating system for
                     applications where it might matter.
   ---------------------------------------------------------------- 
*/

msgbox( "This program is not meant to be executed, "+;
        "but contains useful dBL Code.","Don't run this!", 16 )
return


/*
     keyExists()
     Author...: Gary White, Ken Mayer
     Date.....: November 24, 2004
     Notes....: The database objects in dBASE do not have
                a direct equivalent to the Keymatch() function
                in the XDML syntax. This is an attempt to
                give the functionality to the OODML of dBASE.

                KeyExists() checks to see if a specific value
                exists in a table, without moving the row pointer
                in the current table. 
    Usage.....: keyExists( <database>, <table>, <indextag>, <keytofind> )
    Examples..: ? keyExists( "DBASESAMPLES", "FISH", ;
                             "PRIMARYNAME", "Yellowtail Snapper" )
                ? keyExists( null, "MyTable", "MyTag", 5 )
    Parameters: database -- the database alias -- if none, use
                            null
                table    -- name of the table
                indextag -- tag to search in
                keytofind -- the value to search for in the index
    Returns...: logical -- true or false -- true if it exists,
                           false if it doesn't.
*/
function keyExists
   parameters cAlias, cTable, cIndex, xKey
   local d, q, bFound
   // Set up the database alias, if one was passed:
   if not empty( cAlias )
      d = new database()
      // catch it and return an error if it doesn't exist
      try
         d.databaseName := cAlias
         d.active       := true
      catch( Exception E )
         msgbox( "Database Alias: '"+cAlias+"' does not exist!",;
                 "Keymatch Error", 16 )
         return false
      endtry
   endif

   // set up the query:
   q=new query()
   if not empty( cAlias )
      q.database := d
   endif
   // catch error(s) if possible ...
   try
      q.sql    := "select * from '"+cTable+"'"
      q.active := true
   catch( Exception E )
      msgbox( "Table: '"+cTable+"' does not exist!",;
              "Keymatch Error", 16 )
      return false
   endtry

   // set the indexTag:
   try
      // set index:
      q.rowset.indexName := cIndex
   catch( Exception E )
      msgbox( "Index: '"+cIndex+"' does not exist!",;
              "Keymatch Error", 16 )
      return false
   endtry

   // see if value exists ...
   bFound = q.rowset.findKey( xKey )

   // cleanup code
   q.active := false
   release object q
   q=null
   // can't release it if it wasn't created
   if not empty( cAlias )
      d.active := false
      release object d
      d = null
   endif
return bFound
// end of function: keyExists()


/*
   GetDesktopFolder
   Gary White
   Written some time ago, reposted by J.P. Martel in the
   dBASE newsgroups in April, 2006 as an alternative
   to the WSH version of this, which may not work on
   some machines (especially older OSes). 

   Returns the path to the DesktopFolder, which can be handy
   for adding your own icons to the desktop, etc.


   Usage:   ? GetDesktopFolder()
*/
Function GetDesktopFolder
   #include winreg.h
   local cDesktopKey, r, f
   cDesktopKey = 'Software\Microsoft\Windows\CurrentVersion\' + ;
      'Explorer\Shell Folders'
   try
      set proc to :dBASESamples:registry.prg additive
      r = new registry(HKEY_CURRENT_USER, cDesktopKey)
      f = r.queryValue('Desktop')
      if r.error # 0
         f = ''
      endif
      release object r
   catch(exception e)
      msgbox(e.message, 'Error', 16)
      f = ''
   endtry
return f
// End of function: GetDesktopFolder


/*
     roundPrec()
     Author...: Ken Chan, updated by Rich Muller to test for an empty
                value ...
     Date.....: November 23, 2004
     Notes....: This helps deal with the 'rounding error' issues
                in dBASE. Note that while this is not *really*
                a bug in dBASE, and more of an issue with the
                underlying C++ code that dBASE is written in,
                it can be annoying, and the round() function
                built into dBASE often returns results that are
                not truly desirable.

                Ken Chan notes: "I seem to recall some cases which still
                do not behave as expected. It should be noted that
                it (this function) relies on SET PRECISION -- which --
                as always, should be as low as possible, but not lower
                than nPlace (although given the minimum PRECISION
                of 10, that's usually not a problem)."
    Usage.....: roundPrec( <nArg>, <nPlace> )
    Parameters: nArg   = numeric value to round
                nPlace = decimal places to round off at
    Returns...: numeric -- 0 if empty parameter ...
*/
function roundPrec( nArg, nPlace )
   if nPlace <= 0 or nArg == 0
      return round( nArg, nPlace )
   endif
   if empty(nArg)
      return 0.00
   endif
   local nAdj
   if (int( nArg * 10^nPlace ) + iif( nArg > 0, 0.5, -0.5 )) ;
      / 10^nPlace == nArg
      nReturn = round( nArg + sign( nArg )/10^(nPlace+1), nPlace )
   else
      nReturn = round( nArg, nPlace )
   endif
return nReturn
// end of function: roundPrec()

function WhatOS
/*
     WhatOS()
     Author: Rich Muller
     Posted on dBASE Newsgroups by Ronnie MacGregor, Feb. 8, 2006
     Returns a string used to determine which operating system
     an application is being used on ... slightly modified
     by Ken Mayer.

     Return   Operating System     Value returned by OS()
     95     = Windows 95           Windows version 4.00
     98     = Windows 98           Windows version 4.10 
     ME     = Windows ME           Windows version 4.90
     NT3    = Windows NT 3.51      Windows NT version 3.51
     NT4    = Windows NT 4.0       Windows NT version 4.00
     2000   = Windows Server 2000  Windows NT version 5.00
     XP     = Windows XP           Windows NT version 5.01
     2003   = Windows Server 2003  Windows NT version 5.02
*/
   private cOS, cReturn
   cOS = os()
   do case
      case "4.00" $ cOS and not "NT" $ cOS
           cReturn = "95"
      case "4.10" $ cOS and not "NT" $ cOS
           cReturn = "98"
      case "4.90" $ cOS and not "NT" $ cOS
           cReturn = "ME"
      case "3.51" $ cOS and "NT" $ cOS
           cReturn = "NT3"
      case "4.00" $ cOS and "NT" $ cOS
           cReturn = "NT4"
      case "5.00" $ cOS and "NT" $ cOS
           cReturn = "2000"
      case "5.01" $ cOS and "NT" $ cOS
           cReturn = "XP"
      case "5.02" $ cOS and "NT" $ cOS
           cReturn = "2003"
      otherwise
           cReturn = "Error"
   endcase
return cReturn


/*
   -----------------------------------------------------------------------
   End of File: MISCCODE.PRG
   ----------------------------------------------------------------------- 
*/


