/*
  -----------------------------------------------------------------
    Description:
 
      Quick_date is basically an entryfield custom class
		This class contains just one utility to get dates quickly from end_user
		Just the month and day should be sufficient the class will add the current 
		year automatically to the date
		
		It will function with dmy and mdy date formats.
		It will adapt the system date format.
		Quick_date.wfm is a sample test form
		 
 
    Programmers: 
 
       Akshat Kapoor (With a lot of Help and suggestions from Mervyn Bick)
 
    History: 
 
		11th May 2018 Sharing the ready to deploy code for the first time.
      22nd April 2020 Added the method assign_val()

    Instantiation/use: 
 
       set procedure to quick_date.cc additive
		 And then just drag and drop the quick_date entryfield from custom tab 
		 of component palette onto the form
		 
		 form.quick_date1.value will give the character value of entryfield
		 form.quick_date1.date will give the actual date (if valid) entered by the user
		 
		 To assign values two lines will be required
		 form.quick_date1.value = ctod(date()) //or any other date default being an empty date
		 form.quick_date1.date = date()  //or any other date default being an empty date
       
       Using Assign_val the above two lines can be combined into 1
       form.quick_date1.assign_val(date())
		 
		 The following properties may be of a greater help
		 
		 rangemax/rangemin  Type date for setting the max/min values
		 validErrorMsg  Type Character like any standard validErrorMsg
		 
		 This Class uses the onLostFocus & onKey events of the entryfield to 
		 check for valid date / adding of current year etc. 
		 So if for any reason you need to reassign the events then include the 
		 following line as the first line in the customised event
		 
       quick_date::quick_date_onLostFocus() 
		 quick_date::quick_date_onKey()

		 for Any further suggestions please email me  akshatkapoor76 at gmail dot com
   ----------------------------------------------------------------
*/
class quick_date(parentObj) of ENTRYFIELD(parentObj) custom
   with (this)
      onGotFocus = class::quick_date_ONGOTFOCUS
      onLostFocus = class::quick_date_ONLOSTFOCUS
      onKey = class::quick_date_ONKEY
      height = 1.5
      left = 8.0
      top = 7.5
      width = 11.0
      metric = 0	// Chars
      picture = "99"+substr(dtoc(date()),3,1)+"99"+substr(dtoc(date()),3,1)+"9999"
      value = "  "+substr(dtoc(date()),3,1)+"  "+substr(dtoc(date()),3,1)+"    "
		date = ctod("  "+substr(dtoc(date()),3,1)+"  "+substr(dtoc(date()),3,1)+"    ")
   endwith

   function quick_date_onGotFocus()
//      this.keyboard( "{Home}" )
      //unComment the above line if you do not want the entire date to be selected
      return
		
    function assign_val()
      parameters new_val
      if type("new_val") = "D"
         this.value = dtoc(new_val)
         this.date  = new_val
      endif
   return

   function quick_date_onLostFocus()
		if this.value = "  " + substr(dtoc(date()),3,1) + "  "+ substr(dtoc(date()),3,1)+"    "
			//do nothing we cannot perform any validity checks on a blank date
		else
			temp = rtrim(ltrim(this.value)) //remove the spaces
			if len(temp) = 6 //the value entered is dd.mm. 
				temp = temp+ right(dtoc(date()),4) //append the year to the string
				if substr(temp,5,1) = " "
					//to convert 31.3 to 31.03
					temp = left(temp,3) + "0" + substr(temp,4,1)+right(temp,5)
				endif
			endif
			if len(temp) = 8
				temp = left(temp,6)+ substr(dtoc(date()),7,2) + right(temp,2)
				//the entered date is dd.mm.yy insert the century
			endif
			if len(temp) = 10
//				?"We have exact length",temp
				//just to indicate that we have achieved correct length 
				//this loop is not required
			endif
			if dtoc(ctod(temp)) = temp
				this.validDate = true
				this.date = ctod(temp)
//				?"valid date"
				//Here what have I done is try to convert the string to date and convert it back to string
				//if the two values are the same then the date entered is valid otherwise it is not
				//Since we are not checking for month or date ranges at specific location 
				//hence this is independent of the date format
				//Can be used with both dd.mm.yyyy and mm.dd.yyyy format dates
				//For yyyy.mm.dd this type of entryfield is not required
				this.value = temp
				if (type('this.rangemax') = 'D' and ctod(this.value)> this.rangemax)
					if type('this.validerrormsg') = 'C'
						msgbox("*"+this.validerrormsg)
					else
						msgbox("Please enter a value less than or equal to " + dtoc(this.rangemax))
					endif
					this.value = dtoc(this.rangemax)
					this.setfocus()
				endif
				if (type('this.rangemin') = 'D' and ctod(this.value)< this.rangemin)
					if type('this.validerrormsg') = 'C'
						msgbox("***"+this.validerrormsg)
					else
						msgbox("Please enter a value greater than or equal to " + dtoc(this.rangemin))
					endif
					this.value = dtoc(this.rangemin)
					this.setfocus()
				endif
			else
				msgbox("invalid date")
				this.value = dtoc(ctod(temp))
				this.validDate = false
				this.date = ctod("  "+substr(dtoc(date()),3,1)+"  "+substr(dtoc(date()),3,1)+"    ")
				this.setfocus()
				//for best results we should uncomment the above line so that 
				//user is not able to exit the entryfield without a correct date
				//But problem arises if we try to close the form without entering a correct date
				//alternative could be to set the date to blank
			endif
		endif
      return

	function quick_date_onKey(nChar, nPosition,bShift,bControl)
		if (nchar = asc(substr(dtoc(date()),3,1)) or nchar = 32) and nposition = 2
			//the user has pressed . after pressing a single number
			//converting 1 to 01
			this.value = "0" + left(this.value,1) + right(this.value,8)
			this.keyboard( "{Home}{rightarrow}{rightarrow}" )
		endif
		if (nchar = asc(substr(dtoc(date()),3,1)) or nchar = 32) and nposition = 5
			//the user has pressed . after pressing a single number in xx.?
			//converting 12.1 to 12.01
			this.value = left(this.value,3) + "0" + substr(this.value,4,1) + right(this.value,5)
			this.keyboard( "{Home}{rightarrow}{rightarrow}{rightarrow}{rightarrow}" )
		endif
      return
endclass
