//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

   /*


	Classname   :  rmCursor.cc

   Version     :  1.05.1201

   Date        :  Dec 2005


   For         :  dBASE Plus / dB2K / Win9x / NT / Win2000 / WinXP


   Author      :  Ronnie MacGregor

   Contact     :  rmCursorFeedback@dBASEdeveloper.co.uk

   Copyright   :  Ronnie MacGregor   All rights reserved.


   Disclaimer  :  It is the duty of the user of this code to determine its 
                  suitability for use.

                  The use of this code signifies agreement to the following
                  condition of use.
                  
                  In no circumstance will the author accept any responsibility
                  for any consequence arising from the use of this code.


   Description :  A custom control which can be dropped onto a form and used to
                  provide non native cursors for form controls, sourced from 
                  either .cur or .ani files, or cursor resources from a dll.

                  The control has an associative array, "aCursorBank",  which
                  can be loaded with as many cursors as you need in your
                  application, and then the curser for a given control can be
                  set at any given point in time to any one of the cursors in
                  the array referenced simply by name.

                  To check that you have the latest version of this control or
                  if you have the cc but not the demo form, .cur, .ani and dll
                  files, go to:

                     www.dBASEdeveloper.co.uk 


   Useage      :  Use the LoadDLL() method to prepare a resource file for use
                  if required.

                  Use AddCursorFromFile() or AddCursorFromResource() to add
                  cursors to the control array.

                  Use SetNewCursor() at any time to set a cursor for any control.

                  Optionally use RestoreCursor() when the mouse leaves the
                  confines of the control. MouseOver detection, preferrably with
                  mouse capture, is required if you wish to do this, but better
                  overall control is then achieved.

                  Some ActiveX controls need to be treated differently, in that
                  it is the MouseMove event of the NativeObject that needs to be
                  coded rather than the onMouseMove of the dBase ActiveX
                  container object.

                  See rmCursorTest.wfm for sample useage and code.


   Dependancies:

      None


   Acknowledgements:

      Basic method of using a cursor from a file - Gary White

      LoadDLL() - Marc Van den Berghen


   Revisions:

      None


   */

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

class rmCursor (oP) of PaintBox(oP) custom

   // -------------------------------------

   // API externs

   if type("GetCursor") # "FP"      
      extern chandle GetCursor(;
         cvoid                 ;
         ) User32              ;
      from 'GetCursor'
   endif

   if type('LoadCursor') # 'FP'
      extern chandle LoadCursor(;
         chandle,               ; // HINSTANCE hInstance
         cstring                ; // LPCTSTR lpCursorName
         ) user32               ;
      from 'LoadCursorA'
   endif

   if type('LoadCursorFromFile') # 'FP'
      extern chandle LoadCursorFromFile(;
         cstring                        ;
         ) user32                       ;
      from 'LoadCursorFromFileA'
   endif

   if type('LoadLibrary') # 'FP'
      extern chandle LoadLibrary(;
         cstring                 ;
         ) kernel32              ;
      from 'LoadLibraryA'   
   endif

   if type("ReleaseCapture") # "FP"
      extern clogical ReleaseCapture(;
         cvoid                         ;
         ) user32                      ;
      from 'ReleaseCapture'
   endif

   if type("SetCapture") # "FP"
      extern chandle SetCapture(;
         chandle                  ;
         ) User32                 ;
      from 'SetCapture'
   endif

   if type("SetCursor") # "FP"
      extern chandle SetCursor(;
         chandle               ;
         ) User32              ;
      from 'SetCursor'
   endif

   // -------------------------------------

   // Stock Properties
   this.width = 20
   this.height = 20
   this.visible = false

   // -------------------------------------

   // Custom Properties
   this.aCursorBank = new assocArray()

   // -------------------------------------

   function LoadDLL(cName)

      /*

      Author:      Marc Van den Berghen
      Date:		    May 2005
      
      */

      // Load dll to be used for resources
      try                                               
         this.hModule = LoadLibrary(cName)
      catch (exception e)	
         this.hModule = 0
      finally 
         if this.hModule == 0
            msgbox("Error during Resource Initialisation", "Warning", 0)
         endif
      endtry 

   return        

   // -------------------------------------

   function AddCursorFromFile(cCursorName, cFileName)
 
      this.aCursorBank[cCursorName] = LoadCursorFromFile(cFileName)
  
   return

   // -------------------------------------

   function AddCursorFromResource(cCursorName, cResourceName)

      this.aCursorBank[cCursorName] = LoadCursor(this.hModule, cResourceName)

   return

   // -------------------------------------

   function SetNewCursor(oRef, cCursorName)

      oRef.hOldCursor = SetCursor(this.aCursorBank[cCursorName])

   return

   // -------------------------------------

   function RestoreCursor(oRef)

      SetCursor(oRef.hOldCursor)

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

// eof
