//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------


class rmMouseEvents(oP) of PaintBox(oP) custom

   this.version = "1.05.10.15"

   /*

   rmMouseEvents.cc 

   A custom control which provides the ability to programmatically generate 
   mouse events at given screen or form co-ordinates (pixels).


   For         :  dBASE Plus 2.5 and later / Win9x / Win2000 / XP


   Author      :  Ronnie MacGregor


   Contact     :  rmMouseEventsFeedback@dBASEdeveloper.co.uk

   Copyright   :  Ronnie MacGregor   All rights reserved.

   Disclaimer  :  It is the duty of the user of this code to determine its 
                  suitability for use.

                  The use of this code signifies agreement to the following
                  condition of use.
                  
                  In no circumstance will the author accept any responsibility
                  for any consequence arising from the use of this code.
 


   Notes :

      Form Metric must be pixels for the Form*() functions to work.

      This control relies on the use of _app.ExecuteMessages() to ensure that
      mouse event messages and other calls are processed with correct timing.
      It is therefore not recommended for use with versions of dBASE less than
      PLUS 2.5. The control has been coded however to allow it to work with
      lesser versions, but developers should be aware that the accuracy of
      cursor positioning when using any of the "Form*" functions may not be good
      and correction constants may need tweaking in the MouseEventsOnOpen code
      to suit the OS in use. The present values are intended for an application
      being run in the shell, and will be different otherwise.

      Much of the code used here has been pulled from other controls or apps and
      has been grouped together in this cc to provide drop on form mouse event
      functionality in response to a request in the newsgroups from Yiannis H.
      Economides in Cyprus. It has only had minimal testing in this form. 


   Revisions :

      None


   */


   // -------------------------------------


   if type("GetCursorPos") # "FP"
      extern CLOGICAL;
         GetCursorPos(;
            CPTR; //LPPOINT
            );
      user32
   endif


   if type("mouse_event") # "FP"
      #define MOUSEEVENTF_MOVE        0x0001 // mouse move 
      #define MOUSEEVENTF_LEFTDOWN    0x0002 // left button down 
      #define MOUSEEVENTF_LEFTUP      0x0004 // left button up 
      #define MOUSEEVENTF_RIGHTDOWN   0x0008 // right button down 
      #define MOUSEEVENTF_RIGHTUP     0x0010 // right button up 
      #define MOUSEEVENTF_MIDDLEDOWN  0x0020 // middle button down 
      #define MOUSEEVENTF_MIDDLEUP    0x0040 // middle button up 
      #define MOUSEEVENTF_ABSOLUTE    0x8000 // absolute move
      extern CVOID;
         mouse_event(;
            CINT,;      // DWORD dwFlags
            CINT,;      // DWORD dx
            CINT,;      // DWORD dy
            CINT,;      // DWORD dwData
            CPTR;       // ULONG_PTR dwExtraInfo
            );
         user32
   endif


   if type("SetCursorPos") # "FP"
      extern CLOGICAL;
         SetCursorPos(;
            CINT,;
            CINT;
            );
         user32
   endif


   if type("ShowCursor") # "FP"
      extern CINT;
         ShowCursor(;
            CLOGICAL;
            );
         user32
   endif


   // -------------------------------------
   // -------------------------------------

   // Stock properties
   with (this)
      onOpen = CLASS::MouseEventsOnOpen
      visible = false
      width = 20
      height = 20
   endwith

   // -------------------------------------
   // -------------------------------------

   // Custom Properties
   try
      _app.ExecuteMessages()
      this.CanExecuteMessages = true
   catch ( Exception e )
      // Not available in versions prior to 2.5
      this.CanExecuteMessages = false
   endtry

   // -------------------------------------
   // -------------------------------------

   function MouseEventsOnOpen()

      // Establish relationship between screen coords and form client area coords
      // and create x and y correction constants as custom form properties.

      // Get pointer to original form onMouseMove (if any)
      local oP
      oP = form.onMouseMove

      // Substitute our temporary onMouseMove code
      form.onMouseMove = {|flags, col, row|; form.xCorrection = 10 - col; form.yCorrection = 150 - row}

      // Get the current Cursor position
      local sPoint
      sPoint = new Point()
      GetCursorPos(sPoint)

      /*
      Move Cursor to a point that should be within the form
      but so close to the edge that hopefully it won't be over
      another control. The act of moving the cursor should fire
      our substituted onMouseMove code, and create coord correction
      factor custom properties of the form.
      */
      form.Mouse.ScreenMoveMouseTo(form.left + 10, form.top + 150)

      // Force message processing otherwise the restoring of the onMouseMove
      // will happen before the substituted onMouseMove event fires.
      try
         _app.ExecuteMessages()
      catch ( Exception e )
         /*
         _app.ExecuteMessages() Not available in versions prior to 2.5
         Set basic approximate correction factors - better than nothing.
         These are for running in the shell and will need to be changed otherwise.                
         */
         if "5.01" $ OS() // XP
            form.xCorrection = 4 + 2
            form.yCorrection = 30 + 78
         else
            form.xCorrection = 4 + 8
            form.yCorrection = 23 + 76
         endif
      endtry

      // Restore old form onMouseMove if any
      form.onMouseMove := oP

      // Put Mouse back where we got it
      form.Mouse.ScreenMoveMouseTo(sPoint.GetX(), sPoint.GetY())

   return   

   // -------------------------------------

   function ScreenMoveMouseTo(x, y)

      // Place cursor at point
      SetCursorPos(x, y)

   return

   // -------------------------------------

   function ScreenLeftClick(x, y)

      // Place cursor at start point
      SetCursorPos(x, y)

      // Mouse button down
      mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, null)

      // Mouse button up
      mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, null)
 
      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function ScreenRightClick(x, y)

      // Place cursor at start point
      SetCursorPos(x, y)

      // Mouse button down
      mouse_event(MOUSEEVENTF_RIGHTDOWN, 0, 0, 0, null)

      // Mouse button up
      mouse_event(MOUSEEVENTF_RIGHTUP, 0, 0, 0, null)
 
      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function ScreenMiddleClick(x, y)

      // Place cursor at start point
      SetCursorPos(x, y)

      // Mouse button down
      mouse_event(MOUSEEVENTF_MIDDLEDOWN, 0, 0, 0, null)

      // Mouse button up
      mouse_event(MOUSEEVENTF_MIDDLEUP, 0, 0, 0, null)
 
      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function ScreenLeftDoubleClick(x, y)

      // Place cursor at start point
      SetCursorPos(x, y)

      // Mouse button down
      mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, null)

      // Mouse button up
      mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, null)

      // Mouse button down
      mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, null)

      // Mouse button up
      mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, null)

      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function FormMoveMouseTo(x, y)

      // Place cursor at point
      SetCursorPos(form.left + form.xCorrection + x, form.top + form.yCorrection + y)

   return

   // -------------------------------------

   function FormLeftClick(x, y)

      // Place cursor at start point
      SetCursorPos(form.left + form.xCorrection + x, form.top + form.yCorrection + y)

      // Mouse button down
      mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, null)

      // Mouse button up
      mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, null)
 
      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function FormRightClick(x, y)

      // Place cursor at start point
      SetCursorPos(form.left + form.xCorrection + x, form.top + form.yCorrection + y)

      // Mouse button down
      mouse_event(MOUSEEVENTF_RIGHTDOWN, 0, 0, 0, null)

      // Mouse button up
      mouse_event(MOUSEEVENTF_RIGHTUP, 0, 0, 0, null)
 
      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function FormMiddleClick(x, y)

      // Place cursor at start point
      SetCursorPos(form.left + form.xCorrection + x, form.top + form.yCorrection + y)

      // Mouse button down
      mouse_event(MOUSEEVENTF_MIDDLEDOWN, 0, 0, 0, null)

      // Mouse button up
      mouse_event(MOUSEEVENTF_MIDDLEUP, 0, 0, 0, null)
 
      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function FormLeftDoubleClick(x, y)

      // Place cursor at start point
      SetCursorPos(form.left + form.xCorrection + x, form.top + form.yCorrection + y)

      // Mouse button down
      mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, null)

      // Mouse button up
      mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, null)

      // Mouse button down
      mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, null)

      // Mouse button up
      mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, null)

      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function FormSelectDrag(xStart, yStart, xEnd, yEnd)

      // Place cursor at start point
      SetCursorPos(form.left + form.xCorrection + xStart, form.top + form.yCorrection + yStart)

      // Mouse button down
      mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, null)

      // Use this to initiate mouse move, but not to do the actual moving
      // since correction would be needed for mouse acceleration, screen 
      // resolution, etc. etc. ...
      mouse_event(MOUSEEVENTF_MOVE, 0, 0, 0, null)

      // Move mouse 
      SetCursorPos(form.left + form.xCorrection + xEnd, form.top + form.yCorrection + yEnd)

      // Mouse button up
      mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, null)

      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function LeftMouseDown()

      // Mouse button down
      mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, null)
 
      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function LeftMouseUp()

      // Mouse button up
      mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, null)
 
      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------


   function RightMouseDown()

      // Mouse button down
      mouse_event(MOUSEEVENTF_RIGHTDOWN, 0, 0, 0, null)
 
      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function RightMouseUp()

      // Mouse button up
      mouse_event(MOUSEEVENTF_RIGHTUP, 0, 0, 0, null)
 
      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------


   function MiddleMouseDown()

      // Mouse button down
      mouse_event(MOUSEEVENTF_MIDDLEDOWN, 0, 0, 0, null)
 
      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function MiddleMouseUp()

      // Mouse button up
      mouse_event(MOUSEEVENTF_MIDDLEUP, 0, 0, 0, null)
 
      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

   function RelativeMove(x, y)

      // Get the current Cursor position
      local sPoint
      sPoint = new Point()
      GetCursorPos(sPoint)

      // Use this to initiate mouse move, but not to do the actual moving
      // since correction would be needed for mouse acceleration, screen 
      // resolution, etc. etc. ...
      mouse_event(MOUSEEVENTF_MOVE, 0, 0, 0, null)

      // Move mouse 
      SetCursorPos(sPoint.GetX() + x, sPoint.GetY() + y)

      // Process mouse message
      if this.CanExecuteMessages == True
         _app.ExecuteMessages()
      endif

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

class Point of struct

   /*

   Author:      Gary White
   Date:		    September 16, 2000

	Description: Basic structures for use in api calls.
      
   */

	this.string := space(4)

   function getX
   	return int(this.getLong(0))

   function setX(n)
   	return this.setLong(0, n)

   function getY
   	return int(this.getLong(4))

   function setY(n)
   	return this.setLong(4, n)

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

class Struct of string

   /*

   Author:      Gary White
   Date:		    September 16, 2000

	Description: Generic structure class used as superclass for other structs.

   Modified by Ronnie MacGregor and Marc Van den Berghen May 2005
      
   */

   // -------------------------------------

   function getWord(nIndex)
      // This function added by Ronnie MacGregor - May 2005 
   	local n
      n = this.getByte(nIndex) + ;
      	bitlshift(this.getByte(nIndex + 1), 8) 
   return n

   // -------------------------------------

   function getDWord(nIndex)
      // This function added by Ronnie MacGregor - May 2005
      // With information from Marc Van den Berghen   
   	local n
      n = this.getByte(nIndex) + ;
      	bitlshift(this.getByte(nIndex + 1), 8) + ;
      	bitlshift(this.getByte(nIndex + 2), 16) + ;
      	bitlshift(this.getByte(nIndex + 3), 24)
   return n

   // -------------------------------------

   function getLong(nIndex)
   	local n
      n = this.getByte(nIndex) + ;
      	bitlshift(this.getByte(nIndex + 1), 8) + ;
      	bitlshift(this.getByte(nIndex + 2), 16) + ;
      	bitlshift(this.getByte(nIndex + 3), 24)
      if bitset(n, 31)
         // This correction by Marc Van den Berghen May 2005 
         // n := - bitnot(n) // (Gary's original line)
         n := - bitnot(n) - 1 
      endif
   return n

   // -------------------------------------

   function setLong(nIndex, nValue)
   	local bRet
   	try
      	bRet = true
         this.setByte(nIndex, bitand(nValue, 0xFF))
         this.setByte(nIndex + 1, bitrshift(bitand(nValue, 0xFF00), 8))
         this.setByte(nIndex + 2, bitrshift(bitand(nValue, 0xFF0000), 16))
         this.setByte(nIndex + 3, bitrshift(bitand(nValue, 0xFF000000), 24))
      catch(exception e)
      	bRet = false
      endtry
   return bRet

   // -------------------------------------

   function release
      this.string = ''
      release object this
   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------


// eof