/*
   SetEnvironment.prg

   Programmer...: Ken Mayer
   Date.........: April 8, 2002
   Notes........: Based on a request in the dBASE Newsgroups, this
                  is a function that should handle setup of the 
                  developer's environment using the INI class in
                  the dUFLP (which in turn uses some other classes,
                  see below).

                  What this does is handle the custom form and
                  custom report settings in the dB2K.ini file,
                  by getting information stored in a special
                  environment INI file that stores this info.

                  What you need is to call the program from the
                  dUFLP called SaveEnvironment to actually save
                  the current environment first, and then
                  when you change from one working folder to 
                  another, you can call this one to set the 
                  environment ... if no environment.ini exists,
                  then this routine will empty out the custom
                  report and custom form entries in the dB2K.ini
                  file ...

                  The only thing missing is the ability to get
                  dB2K (dBASE Plus, et al) to re-load the 
                  .INI file without shutting down and re-starting
                  the application.

                  Due to the way that dB2K (et al) work, saving
                  the current folder to the .INI file is pointless,
                  for a couple of reasons:
                    1) When the IDE closes, that setting is saved
                       in the .INI file already;
                    2) If this program is being executed from the
                       folder you are setting the environment for,
                       the current folder setting is saved to the
                       .ini file in the first place (duh).

  Usage.........: do :dUFLP:SetEnvironment

  Misc. Notes...: This routine relies on the dUFLP source alias,
                  and the following:

                  INI.CC
                     StringEX.cc
                     SetProc.prg

  Warning.......: This may seem a bit confusing at first, but
                  there is no way to re-load the dB2K.INI file,
                  so what this program is going to do is 
                  do it's job, start a new instance of dB2K,
                  and close the current one.

                  Also note that if using Win 95, 98, or ME,
                  and dB2K, the IDE will not restart, but if you're
                  using NT, Win 2K, Windows XP, etc., and/or
                  dBASE SE, dBASE PLUS, etc., it should start
                  a new instance with the new settings.
*/

set procedure to :duflp:ini.cc additive

// uses *current* folder for environment.ini file
cPath = set("directory")
// note that if the .INI doesn't exist, then ini.cc
// will create it ...
oEnvINI = new ini( cPath+"\environment.ini" )
cSection = "Custom"
cCustFormFile = oEnvINI.getValue( cSection, "FormFile" )
if empty(cCustFormFile) 
  oEnvINI.setValue( cSection, "FormFile", "" ) // in case it doesn't exist
endif
cCustForm = oEnvINI.getValue( cSection, "Form" )
if empty(cCustForm) 
  oEnvINI.setValue( cSection, "Form", "" ) // in case it doesn't exist
endif

cCustReportFile = oEnvINI.getValue( cSection, "ReportFile" )
if empty(cCustReportFile) 
  oEnvINI.setValue( cSection, "ReportFile", "" ) // in case it doesn't exist
endif
cCustReport = oEnvINI.getValue( cSection, "Report" )
if empty(cCustReport) 
  oEnvINI.setValue( cSection, "Report", "" ) // in case it doesn't exist
endif

// read/set the Function Keys:
cSection = "FunctionKeys"
for i = 1 to 12
    cCmd = "cKey = oEnvINI.GetValue( cSection, 'F"+i+"' )"
    &cCmd.
    if not empty( cKey )
       set function i to cKey
    endif
next

// On/Off and other environment settings based
// on some work by Gary White in a class called SaveEnv.cc:
cSection = "OnOffMisc"
cValue = oEnvINI.GetValue( cSection, "Alternate" )
if not empty( cValue ) // if we have nothing here, we need to skip
                   // all of the following
   set alternate &cValue.
   cValue = oEnvINI.GetValue( cSection, "AutoSave" )
   set autosave &cValue.
   cValue = oEnvINI.GetValue( cSection, "AutoNullFields" )
   set autonullfields &cValue.
   cValue = oEnvINI.GetValue( cSection, "Bell" )
   set bell &cValue.
   cValue = oEnvINI.GetValue( cSection, "BellTo" )
   set bell to &cValue.
   cValue = oEnvINI.GetValue( cSection, "BlocksizeTo" )
   cCmd = "set blocksize to "+cValue
   &cCmd.
   cValue = oEnvINI.GetValue( cSection, "Century" )
   set century &cValue.
   cValue = oEnvINI.GetValue( cSection, "Confirm" )
   set confirm &cValue.
   cValue = oEnvINI.GetValue( cSection, "Console" )
   set console &cValue.
   cValue = oEnvINI.GetValue( cSection, "Coverage" )
   set coverage &cValue.
   cValue = oEnvINI.GetValue( cSection, "CUAEnter" )
   set cuaenter &cValue.
   cValue = oEnvINI.GetValue( cSection, "Currency" )
   set currency &cValue.
   cValue = oEnvINI.GetValue( cSection, "CurrencyTo" )
   set currency to cValue // no macro needed
   cValue = oEnvINI.GetValue( cSection, "DatabaseTo" )
   if not empty( cValue )
      open database (cValue)   // have to open it first
      set database to (cValue) 
   endif
   cValue = oEnvINI.GetValue( cSection, "Date" )
   set date &cValue.
   cValue = oEnvINI.GetValue( cSection, "DBTypeTo" )
   set dbtype to &cValue.
   cValue = oEnvINI.GetValue( cSection, "Decimals" )
   set decimals to val( cValue )
   cValue = oEnvINI.GetValue( cSection, "Deleted" )
   set deleted &cValue.
   cValue = oEnvINI.GetValue( cSection, "Design" )
   set design &cValue.
   cValue = oEnvINI.GetValue( cSection, "Development" )
   set development &cValue.
   cValue = oEnvINI.GetValue( cSection, "DirectoryTo" )
   set directory to "&cValue."
   cValue = oEnvINI.GetValue( cSection, "EditorTo" )
   set editor to cValue // no macro
   cValue = oEnvINI.GetValue( cSection, "Encryption" )
   set encryption &cValue.
   cValue = oEnvINI.GetValue( cSection, "EpochTo" )
   set epoch to val( cValue ) // need numeric value
   cValue = oEnvINI.GetValue( cSection, "ErrorTo" )
   set error to cValue // no macro
   cValue = oEnvINI.GetValue( cSection, "Escape" )
   set escape &cValue.
   cValue = oEnvINI.GetValue( cSection, "Exact" )
   set exact &cValue.
   cValue = oEnvINI.GetValue( cSection, "Exclusive" )
   set exclusive &cValue.
   cValue = oEnvINI.GetValue( cSection, "Fields" )
   set fields &cValue.
   cValue = oEnvINI.GetValue( cSection, "FieldsTo" )
   set fields to &cValue.
   cValue = oEnvINI.GetValue( cSection, "FullPath" )
   set fullpath &cValue.
   cValue = oEnvINI.GetValue( cSection, "Headings" )
   set headings &cValue.
   cValue = val( oEnvINI.GetValue( cSection, "HoursTo" ) )
   cCmd = "set hours to "+cValue
   &cCmd.
   cValue = oEnvINI.GetValue( cSection, "IBlockTo" )
   cCmd = "set iblock to "+cValue
   &cCmd.
   cValue = oEnvINI.GetValue( cSection, "LDCheck" )
   set ldcheck &cValue.
   cValue = oEnvINI.GetValue( cSection, "LDConvert" )
   set ldconvert &cValue.
   cValue = oEnvINI.GetValue( cSection, "Lock" )
   set lock &cValue.
   cValue = oEnvINI.GetValue( cSection, "MarginTo" )
   set margin to val( cValue )
   cValue = oEnvINI.GetValue( cSection, "MarkTo" )
   set mark to cValue // no macro
   cValue = oEnvINI.GetValue( cSection, "MBlockTo" )
   set mBlock to val( cValue )
   cValue = oEnvINI.GetValue( cSection, "MessageTo" )
   set message to cValue // no macro
   cValue = oEnvINI.GetValue( cSection, "Near" )
   set near &cValue.
   cValue = oEnvINI.GetValue( cSection, "OdometerTo" )
   set odometer to val( cValue )
   cValue = oEnvINI.GetValue( cSection, "PathTo" )
   set path to cValue // no macro
   cValue = oEnvINI.GetValue( cSection, "PointTo" )
   set point to cValue // no macro
   cValue = oEnvINI.GetValue( cSection, "PrecisionTo" )
   set precision to val( cValue )
   cValue = oEnvINI.GetValue( cSection, "RefreshTo" )
   set refresh to val( cValue )
   cValue = oEnvINI.GetValue( cSection, "ReprocessTo" )
   set reprocess to val( cValue )
   cValue = oEnvINI.GetValue( cSection, "Safety" )
   set safety &cValue.
   cValue = oEnvINI.GetValue( cSection, "SeparatorTo" )
   set separator to cValue // no macro
   cValue = oEnvINI.GetValue( cSection, "Space" )
   set space &cValue.
   cValue = oEnvINI.GetValue( cSection, "Talk" )
   set talk &cValue.
   cValue = oEnvINI.GetValue( cSection, "Title" )
   set title &cValue.
   cValue = oEnvINI.GetValue( cSection, "TypeaheadTo" )
   set typeahead to val( cValue )
   cValue = oEnvINI.GetValue( cSection, "Unique" )
   set unique &cValue.
   cValue = oEnvINI.GetValue( cSection, "SpeedBar" )
   _app.speedBar := iif( cValue=="true", true, false )
   cValue = oEnvINI.GetValue( cSection, "StatusBar" )
   _app.statusBar := iif( cValue=="true", true, false )
endif

oEnvINI.flush()
oEnvINI.release()

// now let's open the dB2K.ini and set values there:
oIDEINI = new ini( true ) // should invoke dBASE INI 
                          // (dB2K.ini, PLUS.ini if using
                          // dBASE Plus, etc.)
cSection = "FormDesigner"
cEntry   = "BaseFormClassFileName"
oIDEIni.SetValue( cSection, cEntry, cCustFormFile )
cEntry   = "BaseFormClassName"
oIDEIni.SetValue( cSection, cEntry, cCustForm )
cSection = "ReportDesigner"
cEntry   = "BaseReportClassFileName"
oIDEIni.SetValue( cSection, cEntry, cCustReportFile )
cEntry   = "BaseReportClassName"
oIDEIni.SetValue( cSection, cEntry, cCustReport )

// Unfortunately, it's not possible to 
// save the current folder in the .INI file,
// as when the software closes, it over-writes
// the setting (tested it, and found out)

// make sure it's closed
oIDEIni.flush()
// release the ini object
oIDEIni.release()

// start a new instance
// NOTE: this will only work on OSes of Win NT, and later
// (i.e., not Windows 95, 98, ME ...) when running dB2K ... 
// it should work fine on dBASE PLUS, dBASE SE ... and so on.
if not empty( _app.exeName )
   run(true,_app.exeName)

   // exit ...
   quit
else

   msgbox( "Environment Set, quit and restart ...", "Set Environment" )

endif



